local off = W_OPTION:Name('Off')
local screen

-- インデックスとオプション名が一致していたらオプションを返す
local function GetOptionWhenIndexMatchName(index, name)
    if not screen then return nil end
    local option = screen:GetOptionRow(index)
    return (option:GetName() == name) and option or nil
end

local function InputKey(params)
    if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
        if params.button == 'Left' then
            MESSAGEMAN:Broadcast('PressKey', {Button = 'Left', Player = params.PlayerNumber})
        end
        if params.button == 'Right' then
            MESSAGEMAN:Broadcast('PressKey', {Button = 'Right', Player = params.PlayerNumber})
        end
    end
end

local pnOptionName = {
    Gimmick = {
        [PLAYER_1] = W_PLAYER:ReadPref(PLAYER_1, 'GimmickName', off),
        [PLAYER_2] = W_PLAYER:ReadPref(PLAYER_2, 'GimmickName', off),
    },
}
local pnOptionConfList = {
    Gimmick = {[PLAYER_1] = {off}, [PLAYER_2] = {off}},
}
local pnOptionConfSel = {
    Gimmick = {[PLAYER_1] = 1, [PLAYER_2] = 1},
}
local targetOptions = {
    Gimmick = 'GimmickConfig',
}
return Def.ActorFrame({
    OnCommand = function(self)
        screen = SCREENMAN:GetTopScreen()
        screen:AddInputCallback(InputKey)
	end,
    -- オプションが変更された
    ChangedOptionMessageCommand = function(self, params)
        local pn = params.Player
        -- 変更したオプション
        local name = params.Name
        local opt = params.Params
        local oldName = pnOptionName[name][pn]
        pnOptionName[name][pn] = opt.Name
        pnOptionConfList[name][pn] = opt.Config
        pnOptionConfSel[name][pn] = 1
        local value = params.SetValue(pn, oldName ~= pnOptionName[name][pn], opt.Default)
        for i,conf in pairs(pnOptionConfList[name][pn]) do
            if string.lower(conf) == string.lower(value) then
                pnOptionConfSel[name][pn] = i
                break
            end
        end
        self:queuecommand(string.format('SetConfigText%s%s', name, pname(pn)))
    end,
    -- P1のギミック設定テキストを変更
    SetConfigTextGimmickP1Command = function(self)
        self:playcommand('SetConfigText', {Player = PLAYER_1, Target = 'Gimmick'})
    end,
    -- P2のギミック設定テキストを変更
    SetConfigTextGimmickP2Command = function(self)
        self:playcommand('SetConfigText', {Player = PLAYER_2, Target = 'Gimmick'})
    end,
    -- ギミック設定テキストを変更
    SetConfigTextCommand = function(self, params)
        local pn = params.Player
        local name = params.Target
        for i=1, screen:GetNumRows()-1 do
            local option = GetOptionWhenIndexMatchName(i, targetOptions[name])
            if option then
                option:GetChild(''):GetChild('Item')[(pn == PLAYER_1) and 1 or 2]
                    :settext(pnOptionConfList[name][pn][pnOptionConfSel[name][pn]])
                -- 出来ればフレームの大きさも変えたいけどやり方がわからない
                break
            end
        end
    end,
    -- ギミック設定での左右キー押下処理
    PressKeyMessageCommand = function(self, params)
        local pn = params.Player
        local optionIndex = screen:GetCurrentRowIndex(pn)
        for name,target in pairs(targetOptions) do
            option = GetOptionWhenIndexMatchName(optionIndex, target)
            if option then
                if params.Button == 'Left' then
                    pnOptionConfSel[name][pn] = pnOptionConfSel[name][pn]-1
                elseif params.Button == 'Right' then
                    pnOptionConfSel[name][pn] = pnOptionConfSel[name][pn]+1
                end
                if pnOptionConfSel[name][pn] < 1 then
                    pnOptionConfSel[name][pn] = #pnOptionConfList[name][pn]
                end
                if pnOptionConfSel[name][pn] > #pnOptionConfList[name][pn] then
                    pnOptionConfSel[name][pn] = 1
                end
                self:playcommand(string.format('SetConfigText%s%s', name, pname(pn)))
            end
        end
    end,
    -- 保存処理
    OffCommand = function(self)
        for _,pn in pairs(PlayerNumber) do
            if W_PLAYER:IsEnabled(pn) then
                for name,target in pairs(targetOptions) do
                    W_PLAYER:SavePref(pn, target, pnOptionConfList[name][pn][pnOptionConfSel[name][pn]])
                end
            end
        end
    end,
    -- サウンドが出ないので対応
    LoadActor(THEME:GetPathS('Common', 'value'))..{
        InitCommand = function(self) W:SE(self) end,
        PressKeyMessageCommand = function(self, params)
            local pn = params.Player
            local optionIndex = screen:GetCurrentRowIndex(pn)
            for _,target in pairs(targetOptions) do
                option = GetOptionWhenIndexMatchName(optionIndex, target)
                if option then
                    self:play()
                end
            end
        end,
    },
})