local screen
local lock = 0.6
local actorSelf
local current = 1
local prefList = {
    OF = 'OutFoxPrefs.ini',
}
local pref = {}
for k,v in pairs(prefList) do
    local path = '/Save/'..v
    if FILEMAN:DoesFileExist(path) then
        pref[k] = IniFile.ReadFile(path)
    end
end
setenv('WAIEI_CUSTOM_PREF', pref)

local configList = {
    {Key = 'MemoryCards', List = {false, true}, Default = false, File = ''},
    {Key = 'CustomSongsEnable', List = {false, true}, Default = false, File = ''},
    {Key = 'ShowThemeErrors', List = {false, true}, Default = false, File = ''},
    {Key = 'SongBackgrounds', List = {false, true}, Default = false, File = ''},
    {Key = 'RateModsAffectFGChanges', List = {false, true}, Default = true, File = ''},
    {Key = 'CustomShadersDisabled', List = {false, true}, Default = false, File = ''},
    {Key = 'DrawHoldsAndTapsInSameLoop', List = {false, true}, Default = false, File = ''},
    {Key = 'GimmickMode', List = {false, true}, Default = true, File = ''},
    {Key = 'AllowVRR', List = {false, true}, Default = true, File = ''},
    {Key = 'SmartTimings', List = {
        'Original', 'Advanced', 'ITG', 'ITG Advanced', 'SMX', 'FAPlus', 'ECFA', 'Taitai',
        'Pump Normal', 'Pump Hard', 'Pump Very Hard', 'KBX', 'DDR', 'BeatmaniaIIDX', "Pop'n Music",
        'GuitarFreaks', 'GuitarHero', 'GuitarH', 'Drummania', 'ParaParaParadise',
    }, Default = 'Original', File = 'OF'},
}

local removeList = {}
for k,v in pairs(configList) do
    if v.File == '' then
        if PREFSMAN:PreferenceExists(v.Key) then
            configList[k].Value = PREFSMAN:GetPreference(v.Key)
            configList[k].Select = 1
            for index, value in pairs(v.List) do
                if value == configList[k].Value then
                    configList[k].Select = index
                    break
                end
            end
        else
            -- そもそも項目が存在しない（非対応本体）の場合は項目から消す
            removeList[#removeList+1] = k
        end
    else
        -- OF用設定
        local data = getenv('WAIEI_CUSTOM_PREF')[v.File] or nil
        if data and data[v.Key] then
            configList[k].Value = data[v.Key]
            configList[k].Select = 1
            for index, value in pairs(v.List) do
                if value == configList[k].Value then
                    configList[k].Select = index
                    break
                end
            end
        else
            removeList[#removeList+1] = k
        end
    end
end
for i=1, #removeList do
    table.remove(configList, removeList[#removeList-(i-1)])
end
W_CUSTOM:SV('configList', configList)

-- キー・マウス操作
local function CallbackInput(params)
    local function CheckButton(params, key)
        return (params.button and params.button == key)
            or (params.GameButton and params.GameButton == key)
    end

    if params.type == 'InputEventType_Release' then
        if CheckButton(params, 'Back') then
            actorSelf:playcommand('EndCustom')
        end
        if CheckButton(params, 'Start') then
            -- 保存の場合は保存処理、それ以外の場合は最下部へ
            if current < #configList+1 then
                current = #configList+1
                actorSelf:playcommand('Cursor')
            else
                actorSelf:playcommand('Save')
            end
        end
    end
    if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
        if CheckButton(params, 'Up') then
            if current > 1 then
                current = current-1
            else
                current = #configList+1
            end
            actorSelf:playcommand('Cursor')
        end
        if CheckButton(params, 'Down') then
            if current < #configList+1 then
                current = current+1
            else
                current = 1
            end
            actorSelf:playcommand('Cursor')
        end
        if CheckButton(params, 'Left') then
            if current > #configList then return end
            if configList[current].Select > 1 then
                configList[current].Select = configList[current].Select-1
            else
                configList[current].Select = #configList[current].List
            end
            configList[current].Value = configList[current].List[configList[current].Select]
            actorSelf:playcommand('Change')
        end
        if CheckButton(params, 'Right') then
            if current > #configList then return end
            if configList[current].Select < #configList[current].List then
                configList[current].Select = configList[current].Select+1
            else
                configList[current].Select = 1
            end
            configList[current].Value = configList[current].List[configList[current].Select]
            actorSelf:playcommand('Change')
        end
    end
end

return Def.ActorFrame({
    InitCommand = function(self)
        actorSelf = self
    end,
    OnCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = current, Init = true})
        screen = SCREENMAN:GetTopScreen()
        screen:lockinput(lock)
        screen:AddInputCallback(CallbackInput)
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Profile')})
    end,
    -- 項目を選択
    CursorCommand = function(self)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('ChangedCursor', {Index = current})
    end,
    -- 設定を変更
    ChangeCommand = function(self)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('UpdateValue', {Index = current, Value = configList[current].Value})
    end,
    -- 設定を保存
    SaveCommand = function(self)
        self:playcommand('PlayStartSound')
        for _,v in pairs(configList) do
            if v.File == '' then
                if PREFSMAN:PreferenceExists(v.Key) then
                    PREFSMAN:SetPreference(v.Key, v.List[v.Select])
                end
            else
                local path = '/Save/'..prefList[v.File]
                local data = getenv('WAIEI_CUSTOM_PREF')[v.File] or nil
                if data and data[v.Key] then
                    data[v.Key] = v.List[v.Select]
                    setenv('WAIEI_CUSTOM_PREF', data)
                    local f = RageFileUtil.CreateRageFile()
                    if f:Open(path, 2) then
                        for iniK,iniV in pairs(data) do
                            f:PutLine(string.format("%s=%s", iniK, tostring(iniV)))
                        end
                        f:Close()
                    end
                    f:destroy()
                end
            end
        end
        self:playcommand('EndCustom', {SE = false})
    end,
    EndCustomCommand = function(self)
        setenv('WAIEI_CUSTOM_PREF', nil)
    end,
    W_CUSTOM:Template('Basic')
})