local fullWidth, fullHeight = ...
fullWidth  = (fullWidth or 256)
fullHeight = (fullHeight or 36)
local width  = 0.1 * fullWidth
local height = 0.1 * fullHeight
local max   = {100, 100, 100,  70,  50,  55,  70, 100,  90,  60}
local min   = { 65,  70,  40,  30,  30,  30,  50,  75,  60,  40}
local cycle = {100, 100, 100, 100,  90,  75,  75,  50,  50,  50}
local spectrumsActor = Def.ActorFrame({})
if not GAMESTATE:IsDemonstration() then
    return spectrumsActor
end
local frame = LoadActor(W:Object('radius'), {
    Id     = 'demo',
    Width  = fullWidth + 10,
    Height = fullHeight + 22,
    Radius = 8,
    Color  = color('0, 0, 0, 0.8'),
})
-- OF5で追加されたAudioVisualizerを使用する場合はParent
local updateMode = W:IsOFV() and 'Parent' or 'Self'

local avList = {}
if updateMode == 'Parent' then
    spectrumsActor[#spectrumsActor+1] = Def.AudioVisualizer({
        -- 何故か一つ多めに取得しないと値が取れない。しかも[1]は0なのでさらに一つ多めに取得が必要
        Amount = 22, -- The amount of colums, its from min of 16 to a max of 128.
        LinearPeaks = true, -- define if we want lenear peaks, as in a slow animation when it goes back to 0.
        PeakHeight = 100, -- the hight of the columns,
        UpdateRate = 0.02, -- the update rate of the columns, the lower the faster, between 10 and 0.01.
        OnCommand = function(self)
            self:visible(false)
            -- Gameplayでは必要っぽい
            self:SetSound(SCREENMAN:GetTopScreen():GetSound())
        end,
        ScreenUpdateMessageCommand = function(self)
            local total = 0
            for i,a in pairs(self:GetQuads()) do
                if i > 1 then   -- 何故か1は0なので無視
                    if i%2 == 0 then
                        total = a:GetZoomedHeight()
                    else
                        avList[math.floor(i/2)] = (total + a:GetZoomedHeight()) / 2
                    end
                end
            end
        end,
    })
end
for i=0, 9 do
    spectrumsActor[#spectrumsActor+1] = LoadActor(W:Object('spectrum'), {
        Width  = width,
        Height = height,
        Max    = max[i+1],
        Min    = min[i+1],
        Cycle  = cycle[i+1],
    })..{
        InitCommand=function(self)
            self:x((i-4.5) * width)
        end,
        ScreenUpdateMessageCommand = function(self)
            if updateMode == 'Parent' then
                self:playcommand('UpdateSpectrum', {
                    Value = avList[i+1] or 0,
                })
            end
        end,
    }
end
return Def.ActorFrame({
    OnCommand = function(self)
        self:x(SCREEN_WIDTH + fullWidth)
        self:y(SCREEN_HEIGHT - fullHeight/2 - 105)
        self:sleep(math.min(W_TIMER:GameJacketSleep(), 1.8))
        self:bouncebegin(0.4)
        self:zoom(0.9)
        --self:x(THEME:GetMetric('ScreenGameplay', 'PlayerP2OnePlayerOneSideX'))
        self:x(SCREEN_WIDTH - 160)
    end,
    frame:Actor()..{
        OnCommand = function(self)
            self:y(9)
        end,
    },
    LoadFont('Common Small Normal')..{
        Text = YA_GROUP:GroupName(GAMESTATE:GetCurrentSong():GetGroupName()),
        OnCommand = function(self)
            self:y(28)
            self:diffuse(0.75, 0.75, 0.75, 1)
            self:zoom(0.5)
            self:maxwidth(fullWidth/0.5)
        end,
    },
    spectrumsActor,
})
