local frameLight = W_OPTION:ReadPref('FrameLight', 'Auto')
local lightSec = 999999
local isHowToPlay = false

local screen
local life = {}
local battery = {}
local lifeColor = {}
local tmpLife = {[PLAYER_1] = 0, [PLAYER_2] = 0}
local oldLife = {[PLAYER_1] = -1, [PLAYER_2] = -1}
local lifeHot = {[PLAYER_1] = false, [PLAYER_2] = false}
local colorCycle = 0

local function PlayerInitActor(pn)
    return Def.ActorFrame({
        ChangedGameplaySongMessageCommand = function(self, params)
            local steps = W:Steps(pn) or params.Steps[pn]
            difficulty = steps:GetDifficulty()
            self:GetParent():GetParent():playcommand('CustomFrameChangedSteps', {
                Player = pn,
                Life = life[pn],
                Battery = battery[pn],
                Song = params.Song,
                Steps = steps,
                Difficulty = difficulty,
            })
        end,
        -- フレアゲージ
        ChangedLifeTypeCommand = function(self, params)
            if not params.Level then return end
            if params.Player == pn then
                -- フレアが無効になった
                if params.Level == 0 and lifeColor[pn] then
                    lifeColor[pn] = nil
                else
                    lifeColor[pn] = params.Color or nil
                end
                self:GetParent():GetParent():playcommand('CustomFrameChangedFlare', {
                    Player = pn,
                    Level = params.Level,
                })
                local lifeValue = life[pn] and oldLife[pn] or 0
                lifeHot[pn] = (lifeValue >= 1.0)
                self:GetParent():GetParent():playcommand('CustomFrameChangedLife', {
                    Player = pn,
                    Life = life[pn] or nil,
                    Battery = battery[pn] or nil,
                    LifeColor = lifeColor[pn] or nil,
                    ColorCycle = colorCycle,
                    Value = lifeValue,
                    Danger = life[pn]
                        and (battery[pn] and (life[pn]:GetLivesLeft() == 1) or life[pn]:IsInDanger())
                        or false,
                    Hot = lifeHot[pn],
                })
            end
        end,
        -- ライフ変動時（虹色用にリアルタイムにもコールする）
        ScreenUpdateMessageCommand = function(self, params)
            if GAMESTATE:IsPlayerEnabled(pn) or (isHowToPlay and pn == PLAYER_2) then
                tmpLife[pn] = life[pn] and life[pn]:GetLife() or 0
                self:GetParent():GetParent():playcommand('CustomFrameUpdate', {
                    Player = pn,
                    Life = life[pn] or nil,
                    Battery = battery[pn] or nil,
                    LifeColor = lifeColor[pn] or nil,
                    ColorCycle = colorCycle,
                    Beat = params.PnBeat[pn],
                    Sec = params.Sec,
                    Hot = lifeHot[pn],
                    LightSec = lightSec,
                    Value = life[pn] and oldLife[pn] or 0,
                })
                if not GAMESTATE:IsPlayerEnabled(W:APn(pn)) and not isHowToPlay then
                    self:GetParent():GetParent():playcommand('CustomFrameUpdate', {
                        Player = W:APn(pn),
                        ColorCycle = colorCycle,
                        Beat = params.PnBeat[W:APn(pn)],
                        Sec = params.Sec,
                        LightSec = lightSec,
                        Value = 0,
                    })
                end
                if tmpLife[pn] == oldLife[pn] then return end
                if math.abs(tmpLife[pn] - oldLife[pn]) < 0.0005 then
                    oldLife[pn] = tmpLife[pn]
                else
                    oldLife[pn] = oldLife[pn] + (tmpLife[pn]-oldLife[pn])/4
                end
                local lifeValue = life[pn] and oldLife[pn] or 0
                lifeHot[pn] = (lifeValue >= 1.0)
                self:GetParent():GetParent():playcommand('CustomFrameChangedLife', {
                    Player = pn,
                    Life = life[pn] or nil,
                    Battery = battery[pn] or nil,
                    LifeColor = lifeColor[pn] or nil,
                    ColorCycle = colorCycle,
                    Value = lifeValue,
                    Danger = life[pn]
                        and (battery[pn] and (life[pn]:GetLivesLeft() == 1) or life[pn]:IsInDanger())
                        or false,
                    Hot = lifeHot[pn],
                })
            end
        end,
        CustomFramePlayerOnCommand = function(self)
            if isHowToPlay then
                life[pn] = screen:GetLifeMeter() or nil
                return
            end
            life[pn] = screen:GetLifeMeter(pn) or nil
            battery[pn] = (life[pn] and life[pn].GetTotalLives) and life[pn]:GetTotalLives() or nil
            local position = screen:GetChild('Overlay'):GetChild('Overlay'):GetChild('Position'):GetChild(pn)
            local graph = screen:GetChild('Underlay'):GetChild('Underlay'):GetChild('Note Graph'):GetChild(pn)
            self:GetParent():GetParent():playcommand('CustomFrameOn', {
                Player = pn,
                Life = life[pn] or nil,
                Battery = battery[pn] or nil,
                Position = position,
                Graph = graph,
            })
        end,
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        isHowToPlay = (W:Song():GetDisplayMainTitle()=="_WAIEI How To Play_")
        screen = SCREENMAN:GetTopScreen()
        self:sleep(0.01)
        self:queuecommand('CustomFramePlayerOn')
    end,
    ScreenUpdateMessageCommand = function(self, params)
        colorCycle = (colorCycle+1)%360
    end,
    ChangedGameplaySongMessageCommand = function(self, params)
        if frameLight == 'Never' then
            lightSec = 999999
        elseif frameLight == 'Always' then
            lightSec = 0
        else
            lightSec = params.Song and params.Song:GetSampleStart() or 999999
        end
        self:GetParent():GetParent():playcommand('CustomFrameChangedSong', {
            Song = W:Song(),
            LightSec = lightSec,
        })
    end,
    PlayerInitActor(PLAYER_1),
    PlayerInitActor(PLAYER_2),
})