local child
local pn = Var 'Player'
local font = getenv('WAIEI_JUDGMENT_FONT') or {}
local fontDir = font.File 
        and string.format('/%sOther/Frame/_judgment/%s/', THEME:GetCurrentThemeDirectory(), font.File)
        or ''
local off = W_OPTION:Name('Off')
local w1Window = PREFSMAN:GetPreference('TimingWindowSecondsW1')
local showPro = not GAMESTATE:IsDemonstration()
                and (
                    GetUserPrefB('UserPrefProtiming' .. ToEnumShortString(pn))
                    or false
                )
                or false
local showFaS = W_PLAYER:ReadPref(pn, 'ShowFastAndSlow', W_OPTION:Name('Off'))
local name = W_OPTION:ReadPref('JudgmentMode')
local prefW1 = PREFSMAN:GetPreference('AllowW1')
local showW1 = (prefW1 == 'AllowW1_Everywhere') or ((prefW1 == 'AllowW1_CoursesOnly') and GAMESTATE:IsCourseMode())

local JudgeCmds = {
    TapNoteScore_W1 = THEME:GetMetric( 'Judgment', 'JudgmentW1Command' ),
    TapNoteScore_W2 = THEME:GetMetric( 'Judgment', 'JudgmentW2Command' ),
    TapNoteScore_W3 = THEME:GetMetric( 'Judgment', 'JudgmentW3Command' ),
    TapNoteScore_W4 = THEME:GetMetric( 'Judgment', 'JudgmentW4Command' ),
    TapNoteScore_W5 = THEME:GetMetric( 'Judgment', 'JudgmentW5Command' ),
    TapNoteScore_Miss = THEME:GetMetric( 'Judgment', 'JudgmentMissCommand' ),
}

local ProtimingCmds = {
    TapNoteScore_W1 = THEME:GetMetric( 'Protiming', 'ProtimingW1Command' ),
    TapNoteScore_W2 = THEME:GetMetric( 'Protiming', 'ProtimingW2Command' ),
    TapNoteScore_W3 = THEME:GetMetric( 'Protiming', 'ProtimingW3Command' ),
    TapNoteScore_W4 = THEME:GetMetric( 'Protiming', 'ProtimingW4Command' ),
    TapNoteScore_W5 = THEME:GetMetric( 'Protiming', 'ProtimingW5Command' ),
    TapNoteScore_Miss = THEME:GetMetric( 'Protiming', 'ProtimingMissCommand' ),
}

local TNSFrames = {
    TapNoteScore_W1 = 0,
    TapNoteScore_W2 = 1,
    TapNoteScore_W3 = 2,
    TapNoteScore_W4 = 3,
    TapNoteScore_W5 = 4,
    TapNoteScore_Miss = 5,
}

local t = Def.ActorFrame ({})
t[#t+1] = Def.ActorFrame ({
    -- 通常判定
    LoadActor(fontDir..name) .. {
        Name = 'Judgment',
        InitCommand = function(self)
            self:pause()
            self:visible(false)
        end,
        OnCommand = THEME:GetMetric('Judgment', 'JudgmentOnCommand'),
        ResetCommand = function(self)
            self:finishtweening()
            self:stopeffect()
            self:xy(0, 0)
            self:visible(false)
        end,
    },
    -- Fast And Slow
    LoadActor(fontDir..'fast_and_slow')..{
        InitCommand = function(self)
            self:visible(false)
            self:animate(false)
            self:xy(0, 25)
        end,
        ShowFaSCommand = function(self, params)
            if params.Player ~= pn or showFaS == off or (
                    params.TapNoteScore ~= 'TapNoteScore_W1'
                    and params.TapNoteScore ~= 'TapNoteScore_W2'
                    and params.TapNoteScore ~= 'TapNoteScore_W3'
                    and params.TapNoteScore ~= 'TapNoteScore_W4'
                    and params.TapNoteScore ~= 'TapNoteScore_W5'
                ) then
                return
            end
            if params.TapNoteScore == 'TapNoteScore_W1'
                and (showFaS ~= 'Pro'
                    or (showFaS == 'Pro' and math.abs(params.Offset) < w1Window/2)
                ) then
                return
            end
            self:finishtweening()
            self:visible(true)
            self:setstate(params.Early and 0 or 1)
            self:diffusealpha(0)
            self:zoomx(2)
            self:zoomy(0)
            self:linear(0.05)
            self:diffusealpha(1)
            self:zoomx(1)
            self:zoomy(1)
            self:sleep(0.45)
            self:linear(0.1)
            self:diffusealpha(0)
            self:zoomx(2)
            self:zoomy(0)
        end,
    },
    -- Proタイミング
    LoadFont('Common Normal') .. {
        Name = 'ProtimingDisplay',
        Text = '',
        InitCommand = function(self)
            self:horizalign(right)
            self:zoom(0.8)
            self:strokecolor(Color('Black'))
            self:visible(false)
        end,
        OnCommand = THEME:GetMetric('Protiming', 'ProtimingOnCommand'),
        ResetCommand = function(self)
            self:finishtweening()
            self:stopeffect()
            self:xy(100, 25)
            self:visible(false)
        end,
    },

    InitCommand = function(self)
        child = self:GetChildren()
    end,

    -- 判定時
    JudgmentMessageCommand=function(self, params)
        -- 異なるプレイヤー
        if params.Player ~= pn then return end
        -- ロングノート
        if params.HoldNoteScore then return end

        self:playcommand('ShowFaS', {
            Player = pn,
            Early = params.Early,
            TapNoteScore = params.TapNoteScore,
            Offset = params.TapNoteOffset,
        })

        local iNumStates = child.Judgment:GetNumStates()
        -- フレーム番号
        local iFrame = TNSFrames[params.TapNoteScore]

        local iTapNoteOffset = params.TapNoteOffset

        if not iFrame then return end

        -- Fast/Slow別判定画像を使用している場合
        if iNumStates == 12 then
            iFrame = iFrame * 2
            if not params.Early then
                iFrame = iFrame + 1
            end
        end

        self:playcommand('Reset')

        -- 通常判定
        child.Judgment:visible( true )
        child.Judgment:setstate( iFrame )
        JudgeCmds[params.TapNoteScore](child.Judgment)

        -- Proタイミング
        if params.TapNoteScore == 'TapNoteScore_Miss' then
            child.ProtimingDisplay:diffuse(0, 0, 0, 0)
        elseif params.TapNoteScore == 'TapNoteScore_W1' then
            child.ProtimingDisplay:diffuse(Color('White'))
        elseif iTapNoteOffset > 0 then
            child.ProtimingDisplay:diffuse(color('#ff33ffff'))
        else
            child.ProtimingDisplay:diffuse(color('#33ccffff'))
        end
        child.ProtimingDisplay:visible( showPro )
        child.ProtimingDisplay:settext( math.floor(iTapNoteOffset * 1000)..' MS' )
        ProtimingCmds[params.TapNoteScore](child.ProtimingDisplay)

    end,
})


return t
