local actors = Def.ActorFrame{}
local width = 110
local height = 18

local gradeList     = {PlayerNumber_P1 = {}, PlayerNumber_P2 = {}}
local fullComboList = {PlayerNumber_P1 = {}, PlayerNumber_P2 = {}}
-- メッセージ
actors.UpdateStepsMessageCommand = function(self, params)
    -- プレイヤーの最高グレード・フルコンボ記録はここで一括取得しておく
    if params.Steps then
        -- プレイヤー分取得
        local player = params.Player
        gradeList[player] = {}
        fullComboList[player] = {}
        local profile = PROFILEMAN:GetProfile(player)
        for _, dif in pairs(Difficulty) do
            -- 順に難易度分取得する（存在有無にかかわらず取得するのでnilチェック必須）
            local steps = params.Song:GetOneSteps(params.Steps:GetStepsType(), dif)
            -- 重すぎて使い物にならない
            -- local steps
            -- if dif == params.Steps:GetDifficulty() then
            --     for _,v in pairs(params.Song:GetStepsByStepsType(params.Steps:GetStepsType())) do
            --         if v:GetHash() == params.Steps:GetHash() then
            --             steps = v
            --             break
            --         end
            --     end
            -- else
            --     steps = params.Song:GetOneSteps(params.Steps:GetStepsType(), dif)
            -- end
            if profile and steps then
                -- ハイスコア一覧
                local highscorelist = profile:GetHighScoreListIfExists(params.Song, steps)
                if highscorelist then
                    local highscores = highscorelist:GetHighScores()
                    -- 一番良いグレード記録
                    local radar = steps:GetRadarValues(player)
                    gradeList[player][dif] = W_SCORE:GetBestGrade(highscores, radar)
                    for h=1, #highscores do
                        -- 一番良いフルコンボ記録
                        local fullCombo = W_SCORE:ScoreFC(highscores[h], radar)
                        if fullCombo > 0 and (not fullComboList[player][dif] or fullComboList[player][dif] > fullCombo) then
                            fullComboList[player][dif] = fullCombo
                        end
                    end
                end
            end
        end
    end
    self:playcommand('CallbackSteps', {
        Song      = params.Song,
        Steps     = params.Steps,
        Player    = params.Player,
        StepsType = params.Steps and params.Steps:GetStepsType() or GAMESTATE:GetCurrentStyle():GetStepsType(),
        Grade     = gradeList,
        FullCombo = fullComboList,
    })
end

-- スタイル
actors[#actors+1] = Def.ActorFrame({
    LoadActor(W:Object('style'), 100, true)..{
        InitCommand = function(self)
            self:y(-height-7)
        end,
    },
})

-- プレイヤーの選択難易度
for key, player in pairs(PlayerNumber) do
    actors[#actors+1] = Def.Quad({
        InitCommand = function(self)
            self:blend('BlendMode_Add')
            self:diffuse(W_PLAYER:Color(player))
            self:zoomto(width/2 + 45, height)
            self:horizalign((player == PLAYER_1) and right or left)
            self:faderight((player == PLAYER_1) and 1 or 0)
            self:fadeleft((player == PLAYER_1) and 0 or 1)
            self:visible(W_PLAYER:IsEnabled(player))
        end,
        CallbackStepsMessageCommand = function(self, params)
            if not W_PLAYER:IsEnabled(player) or not params.Song or not params.Steps or player ~= params.Player then
                return
            end
            self:y(Enum.Reverse(Difficulty)[params.Steps:GetDifficulty()]*height)
        end,
        PlayerJoinedMessageCommand = function(self, params)
            if player == params.Player then
                self:visible(true)
            end
        end,
        PlayerUnjoinedMessageCommand = function(self, params)
            if player == params.Player then
                self:visible(false)
            end
        end,
    })
end


-- プレイヤースコア
actors[#actors+1] = LoadActor('list', width, height)

return actors