local pref = W_OPTION:ReadPref('JacketWheel', false)
local wheelMode = (type(pref) == 'boolean')
    and (pref and 'Jacket' or 'Banner') or (pref or 'Banner')
local isBannerMode = (wheelMode == 'Banner')
local w=192
local h=isBannerMode and 170 or 192

-- IDに一意な値を設定する手段がないので
-- Graphics/Objectsではなく、Wheel用に用意したオブジェクトを使用する
local Banner = LoadActor(W:Object('banner'), {
    Song  = nil,
    Size  = w,
    Ratio = isBannerMode and 256/80 or 192/192,
    Cache = -1,
    Rate  = 0.333333,
    Mode  = isBannerMode and 'Banner' or 'Jacket',
})

local TopBorder = LoadActor(W:Object('radius'), {
    Width  = w,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
    Cell   = {1, 1, 1, 0, 0, 0, 0, 0, 0},
})

local MusicLength = LoadActor(W:Object('radius'), {
    Width  = isBannerMode and 80 or 60,
    Height = 16,
    Radius = 8,
    Color  = {1, 1, 1, 1},
})

local function GetCDPath(song)
    if song then
        if song:HasCDImage() then
            return song:GetCDImagePath(), 'CDImage'
        elseif song:HasJacket() then
            return song:GetJacketPath(), 'Jacket'
        elseif song:HasBanner() then
            return song:GetBannerPath(), 'Banner'
        end
    end
    return THEME:GetPathG('Common fallback', 'jacket'), 'Jacket'
end

-- 背景
local function BgActor()
    if wheelMode == 'CDImage' then
        return Def.Actor({})
    end
    local function BannerActor()
        return Def.ActorFrame({
            -- 上部バー
            TopBorder:Actor()..{
                OnCommand = function(self)
                    self:visible(isBannerMode)
                    self:y(-h/2 + 8)
                end,
                UpdateWheelChildCommand = function(self, params)
                    if not isBannerMode then return end
                    local song = params.Song
                    local col = {1, 1, 1, 0.8}
                    if song then
                        col = YA_GROUP:MenuColor(song)
                    else
                        col = {1, 1, 1, 0.8}
                    end
                    col[4] = params.HasFocus and 1 or 0.75
                    self:playcommand('UpdateRadius', {
                        Color = col,
                    })
                end,
            },
            -- 背景
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w, h - 7)
                    self:vertalign(top)
                    self:y(-h/2 + 7)
                    self:diffuse(0.2, 0.2, 0.25, 0.75)
                end,
            }),
            -- 背景
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w, h - 7)
                    self:vertalign(top)
                    self:y(-h/2 + 7)
                    self:diffuse(0.1, 0.1, 0.1, 0.5)
                end,
                UpdateWheelChildCommand = function(self, params)
                    local song = params.Song
                    local col = {1, 1, 1, 0}
                    if song then
                        col = YA_GROUP:MenuColor(song)
                    else
                        col = {1, 1, 1, 0}
                    end
                    col[4] = 0.5
                    self:diffuse(col)
                    self:diffusebottomedge(0, 0, 0, 0)
                end,
            }),
            -- ジャンルとの区切り線
            Def.Quad({
                InitCommand = function(self)
                    self:y(-h/2 + 140)
                    self:zoomto(w, 1)
                    self:diffuse(color('#666666ff'))
                end,
            }),
        })
    end
    local function JacketActor()
        return Def.ActorFrame({
            -- 背景
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w+18, h+18)
                    self:diffuse(1, 1, 1, 0.5)
                    --self:z(-2)
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w+16, h+16)
                    self:diffuse(0, 0.05, 0.1, 1)
                    --self:z(-2)
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w+12, h+12)
                    self:blend('BlendMode_Add')
                    --self:z(-2)
                end,
                UpdateWheelChildCommand = function(self, params)
                    local song = params.Song
                    local col = {1, 1, 1, 0}
                    if song then
                        col = YA_GROUP:MenuColor(song)
                    else
                        col = {1, 1, 1, 0}
                    end
                    --col[4] = 0.5
                    self:diffuse(col)
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w+8, h+8)
                    --self:z(-2)
                end,
                UpdateWheelChildCommand = function(self, params)
                    local song = params.Song
                    local col = {1, 1, 1, 0}
                    if song then
                        col = YA_GROUP:MenuColor(song)
                    else
                        col = {1, 1, 1, 0}
                    end
                    col[1] = col[1]*0.5
                    col[2] = col[2]*0.5
                    col[3] = col[3]*0.5
                    col[4] = 0.5
                    self:diffuse(col)
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w+6, h+6)
                    --self:z(-2)
                end,
                UpdateWheelChildCommand = function(self, params)
                    local song = params.Song
                    local col = {1, 1, 1, 0}
                    if song then
                        col = YA_GROUP:MenuColor(song)
                    else
                        col = {1, 1, 1, 0}
                    end
                    col[1] = col[1]*0.25
                    col[2] = col[2]*0.25
                    col[3] = col[3]*0.25
                    col[4] = 0.5
                    self:diffuse(col)
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:zoomto(w, h)
                    self:diffuse(0.1, 0.1, 0.1, 1.0)
                end,
            }),
        })
    end
    return Def.ActorFrame({
        -- 影
        LoadActor(W:Object('boxshadow'), 'wheel', {
            Length = 10,
            Color  = {0.1, 0.1, 0.1, 0.4},
            Center = {0.1, 0.1, 0.1, 0.4},
        })..{
            OnCommand = function(self)
                self:y(6)
                self:playcommand('UpdateBoxShadow', {
                    Id         = 'wheel',
                    Width      = w,
                    Height     = h-8,
                    Wait       = 0.5,
                    AnimeAlpha = true,
                })
            end,
        },
        (isBannerMode and BannerActor() or JacketActor()),
    })
end

-- バナー
local function BannerActor()
    if wheelMode == 'CDImage' then
        return Def.ActorFrame({
            InitCommand = function(self)
                self:x(0, -h/2+(isBannerMode and 38 or 96))
            end,
            LoadActor(THEME:GetPathG('cdimage', 'background'))..{
                InitCommand = function(self)
                    self:zoomto(w, w)
                end,
            },
            LoadActor(THEME:GetPathG('cdimage', 'mask'))..{
                InitCommand = function(self)
                    self:zoomto(w, 1.5*w)
                    self:MaskSource(true)
                end,
            },
            -- バナー
            Banner:Actor()..{
                UpdateWheelChildCommand=function(self, params)
                    local path, mode = GetCDPath(params.Song)
                    self:GetParent():playcommand('UpdateBanner', {Path = path})
                    self:y(-h/2+((mode == 'Banner') and 144 or 96))
                    self:MaskDest()
                end,
            },
            LoadActor(THEME:GetPathG('cdimage', 'overlay'))..{
                InitCommand = function(self)
                    self:zoomto(w, w)
                end,
            },
        })
    end
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2+(isBannerMode and 38 or 96))
        end,
        -- バナー
        Banner:Actor()..{
            UpdateWheelChildCommand=function(self, params)
                self:playcommand('UpdateBanner', {
                    Path = isBannerMode
                        and W_GRAPHICS:SongBanner(params.Song)
                        or W_GRAPHICS:SongJacket(params.Song),
                })
            end,
        },
        Def.Quad({
            OnCommand = function(self)
                self:visible(not isBannerMode)
                self:blend('BlendMode_Add')
                self:zoomto(w, h)
                self:diffuseshift()
                self:effectcolor1(1, 1, 1, 0)
                self:effectcolor2(1, 1, 1, 0.1)
                self:effectperiod(2.5)
            end,
        }),
    })
end

-- 曲名
local function SongTitleActor()
    if not isBannerMode then
        return Def.ActorFrame({
            InitCommand = function(self)
                self:xy(0, h/2-12)
            end,
            Def.Quad({
                InitCommand = function(self)
                    self:visible(wheelMode == 'CDImage')
                    self:diffuse(color('0.1, 0.1, 0.15, 0.6'))
                    self:x(-w/2-4)
                    self:zoomto(4, 28)
                end,
                UpdateWheelChildCommand = function(self, params)
                    if wheelMode == 'CDImage' then
                        local song = params.Song
                        local col = {1, 1, 1, 0}
                        if song then
                            col = YA_GROUP:MenuColor(song)
                        else
                            col = {1, 1, 1, 0}
                        end
                        col[4] = 0.75
                        self:diffuse(col)
                    end
                end,
            }),
            Def.Quad({
                InitCommand = function(self)
                    self:diffuse(color('0.1, 0.1, 0.15, 0.6'))
                    self:diffuserightedge(color('0.1, 0.1, 0.1, 0.2'))
                    self:zoomto(w, (wheelMode == 'CDImage') and 28 or 24)
                end,
                UpdateWheelChildCommand = function(self, params)
                    if wheelMode == 'CDImage' then
                        local song = params.Song
                        local col = {1, 1, 1, 0}
                        if song then
                            col = YA_GROUP:MenuColor(song)
                        else
                            col = {1, 1, 1, 0}
                        end
                        col[4] = 0.5
                        self:diffuse(col)
                        self:diffuserightedge(color('0.1, 0.1, 0.1, 0.2'))
                    end
                end,
            }),
            LoadFont('Common Small Normal')..{
                InitCommand=function(self)
                    local textZoom = 0.8
                    self:diffuse(color('#ffffffff'))
                    self:strokecolor(Color('Black'))
                    self:horizalign(left)
                    self:zoom(textZoom)
                    self:maxwidth((w-10)/textZoom)
                    --self:skewx(-0.2)
                    self:x(-w/2+5)
                end,
                UpdateWheelChildCommand=function(self,params)
                    local song = params.Song
                    if song then
                        self:settext(W_STRING:CutWidth(song:GetDisplayMainTitle(), 17))
                    end
                end,
            },
        })
    end
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + 85)
        end,
        -- タイトル
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                self:diffuse(color('#d8d8d8ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:xy(-w/2+10, (wheelMode == 'Jacket') and 40 or 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                local song = params.Song
                if song then
                    local textZoom = 0.9
                    self:zoom(textZoom)
                    self:maxwidth((w-20)/textZoom)
                    self:settext(W_STRING:CutWidth(song:GetDisplayMainTitle(), 13))
                    self:diffusealpha(params.HasFocus and 1 or 0.75)
                end
            end,
        },
    })
end

-- 曲の長さ
local function SongTimeActor()
    if not isBannerMode then
        return Def.ActorFrame({
            InitCommand = function(self)
                self:xy(w/2-40, -h/2+11)
                self:zoom(1.3)
            end,
            MusicLength:Actor()..{
                UpdateWheelChildCommand=function(self, params)
                    local song = params.Song
                    local col = color('0.25, 0.25, 0.3, 0.8')
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        if song:IsMarathon() then
                            col = {0.9, 0.3, 0.3, 0.8}
                        elseif song:IsLong() then
                            col = {0.55, 0.65, 0.3, 0.9}
                        elseif sec < 60 then
                            col = {0.2, 0.75, 1.0, 0.8}
                        end
                    end
                    self:playcommand('UpdateRadius', {
                        Color = col,
                    })
                end,
            },
            LoadFont('Common Small Normal')..{
                InitCommand=function(self)
                    local textZoom = 0.6
                    self:diffuse(color('#ffffffff'))
                    self:zoom(textZoom)
                    self:maxwidth((w/2-20)/textZoom)
                    self:strokecolor(Color('Outline'))
                end,
                UpdateWheelChildCommand=function(self,params)
                    local song = params.Song
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        self:settextf('%d:%02d', math.floor(sec/60), sec%60)
                    end
                end,
            },
        })
    end
    return Def.ActorFrame({
        InitCommand = function(self)
            self:xy(0, -h/2 + (isBannerMode and 115 or 148))
        end,
        -- 長さ
        LoadFont('Common Small Normal')..{
            InitCommand=function(self)
                local textZoom = 0.85
                self:diffuse(color('#a0a0a0ff'))
                self:horizalign(left)
                self:vertalign(top)
                self:zoom(textZoom)
                self:maxwidth((w/2-20)/textZoom)
                self:xy(-w/2+10, 0)
            end,
            UpdateWheelChildCommand=function(self,params)
                self:diffuse(color('#a0a0a0ff'))
                local song = params.Song
                if song then
                    -- 長さソートで不自然にならないように切り捨て
                    local sec = math.floor(song:MusicLengthSeconds())
                    if song:IsMarathon() then
                        self:diffuse(1.0, 0.3, 0.3, 1.0)
                    elseif song:IsLong() then
                        self:diffuse(0.85, 0.95, 0.3, 1.0)
                    elseif sec < 60 then
                        self:diffuse(0.2, 0.75, 1.0, 1.0)
                    end
                    self:settextf('%d:%02d', math.floor(sec/60), sec%60)
                    self:diffusealpha(params.HasFocus and 1 or 0.5)
                end
            end,
        },
        Def.ActorFrame({
            InitCommand=function(self)
                self:xy(w/2-10-40, 6)
            end,
            MusicLength:Actor()..{
                UpdateWheelChildCommand=function(self, params)
                    local song = params.Song
                    local col = color('0, 0, 0, 0')
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        if song:IsMarathon() then
                            col = {0.9, 0.3, 0.3, params.HasFocus and 1 or 0.5}
                        elseif song:IsLong() then
                            col = {0.55, 0.65, 0.3, params.HasFocus and 1 or 0.5}
                        elseif sec < 60 then
                            col = {0.2, 0.75, 1.0, params.HasFocus and 1 or 0.5}
                        end
                    end
                    self:playcommand('UpdateRadius', {
                        Color = col,
                    })
                end,
            },
            LoadFont('Common Small Bold')..{
                InitCommand=function(self)
                    local textZoom = 0.5
                    self:diffuse(color('#ffffffff'))
                    self:zoom(textZoom)
                    self:maxwidth((w/2-20)/textZoom)
                end,
                UpdateWheelChildCommand=function(self,params)
                    local song = params.Song
                    if song then
                        -- 長さソートで不自然にならないように切り捨て
                        local sec = math.floor(song:MusicLengthSeconds())
                        if song:IsMarathon() then
                            self:settext('MARATHON')
                        elseif song:IsLong() then
                            self:settext('LONG')
                        elseif sec < 60 then
                            self:settext('SHORT')
                        else
                            self:settext('')
                        end
                        self:diffusealpha(params.HasFocus and 1 or 0.5)
                    end
                end,
            },
        }),
    })
end

-- ジャンル
local function GenreActor()
    if not isBannerMode then
        return Def.ActorFrame({
            LoadFont('Common Small Normal')..{
                InitCommand = function(self)
                    self:horizalign(right)
                    self:xy(w/2+5, h/2 + 15)
                    self:zoom(0.5)
                    self:maxwidth((w+10)/0.5)
                    self:diffuse(color('#999999ff'))
                    self:strokecolor(color('#00000040'))
                    self:settext('')
                end,
                UpdateWheelChildCommand=function(self, params)
                    local text = ((params.Song and params.Song:GetGenre() ~= '')
                                    and string.upper(params.Song:GetGenre())
                                    or string.upper(ProductFamily()))
                    self:settext(W_STRING:CutWidth(text, 25))
                end,
            },
        })
    end
    return Def.ActorFrame({
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:horizalign(right)
                self:xy(w/2 - 10, -h/2 + 155)
                self:zoom(0.5)
                self:maxwidth((w-20)/0.5)
                self:diffuse(color('#999999ff'))
                self:settext('')
            end,
            UpdateWheelChildCommand=function(self, params)
                local text = ((params.Song and params.Song:GetGenre() ~= '')
                                and string.upper(params.Song:GetGenre())
                                or string.upper(ProductFamily()))
                self:settext(W_STRING:CutWidth(text, 22))
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        if not isBannerMode then
            self:zoom(0.85)
            --self:rotationx(15)
            --self:y(5)
            --self:wag()
            --self:spin()
            --self:effectmagnitude(12.5, 0, 0)
            --self:effectperiod(10)
        end
    end,
    SetCommand=function(self, params)
        -- 多重呼び出しを回避
        if not self.Params then
            self.Params = {Id = math.random(), Song = nil, HasFocus = nil}
            TopBorder:Id('B'..self.Params.Id)
            MusicLength:Id('ML'..self.Params.Id)
        end
        if self.Params and params and params.Song and params.HasFocus ~= nil then
            if params.Song ~= self.Params.Song or params.HasFocus ~= self.Params.HasFocus then
                params.Id = self.Params.Id
                self.Params.Song = params.Song
                self.Params.HasFocus = params.HasFocus
                self:finishtweening()
                self:playcommand('UpdateWheel', params)
            end
        end
    end,
    UpdateWheelCommand = function(self, params)
        if self.Params.Id ~= params.Id then return end
        self:vanishpoint(0, 0)
        self:playcommand('UpdateWheelChild', params)
    end,
    -- 背景
    BgActor(),
    -- バナー
    BannerActor(),
    -- 曲名
    SongTitleActor(),
    -- 長さ
    SongTimeActor(),
    -- ジャンル
    GenreActor(),
    -- デバッグ
    -- LoadFont('Common Normal')..{
    --     UpdateWheelCommand = function(self, params)
    --         local parent = self:GetParent()
    --         if parent.Params.Id == params.Id then
    --             self:settext(params.Song and params.Song:GetDisplayMainTitle()..'\n'..params.Id or 'Group')
    --         end
    --     end,
    -- },
})
