local tc, tcConfig = ...
local lifeWidth = SCREEN_WIDTH/4

-- ライフゲージ
local function LifeActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end

    local function BatteryBorder()
        local a = Def.ActorFrame({})
        for i=1, 9 do
            a[#a+1] = LoadActor('LifeMeterBar frame Border')..{
                OnCommand = function(self)
                    self:visible(false)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if not params.Battery or params.Battery > 10 then
                            self:x(0)
                            self:visible(false)
                        else
                            self:x(-lifeWidth/2 + lifeWidth*i/params.Battery + 1)
                            self:visible(i < params.Battery)
                        end
                    end
                end,
            }
        end
        return a
    end

    local function LifeBar()
        return Def.ActorFrame({
            LoadActor('LifeMeterBar frame left')..{
                OnCommand = function(self)
                    self:x(-lifeWidth/2 - 12)
                end,
            },
            LoadActor('LifeMeterBar frame right')..{
                OnCommand = function(self)
                    self:x(lifeWidth/2 + 12)
                end,
            },
            LoadActor('LifeMeterBar frame Main')..{
                OnCommand = function(self)
                    self:zoomtowidth(lifeWidth)
                end,
            },
            Def.ActorFrame({
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        self:diffusealpha(0.1*(10 - (((params.Beat+0.1)*10)%10)))
                    end
                end,
                LoadActor('LifeMeterBar light left')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:diffuse(tcConfig.Color.Frame)
                        self:x(-lifeWidth/2 - 12)
                    end,
                },
                LoadActor('LifeMeterBar light right')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:diffuse(tcConfig.Color.Frame)
                        self:x(lifeWidth/2 + 12)
                    end,
                },
            }),
            LoadActor('LifeMeterBar scrlight')..{
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:diffuse(tcConfig.Color.Normal)
                    self:zoomtowidth(lifeWidth*2)
                    self:diffuse(tcConfig.Color.Normal)
                end,
                ScreenUpdateWithHSVCommand = function(self, params)
                    if params.Player == pn then
                        local beat2 = 0.1*(params.Beat*10%20)
                        self:cropleft(0.5-(beat2/4))
                        self:cropright(beat2/4)
                        self:x((beat2/4)*(lifeWidth*2)-lifeWidth/2)
                        local curLife = params.Life and params.Life:GetLife() or 0
                        if curLife >= 1.0 then
                            self:diffuse(params.HSV)
                        end
                    end
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        local col = {}
                        if params.Battery and params.Life:GetTotalLives() == 1 then
                            col = tcConfig.Color.Hot
                        elseif params.Value >= 1 then
                            col = tcConfig.Color.Normal
                        elseif params.Danger then
                            col = tcConfig.Color.Danger
                        else
                            col = tcConfig.Color.Normal
                        end
                        self:diffuse(col)
                    end
                end,
            },
            Def.Quad({
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:diffuse(tcConfig.Color.Danger)
                    self:zoomto(lifeWidth-2, 18)
                end,
                ScreenUpdateWithHSVCommand = function(self, params)
                    if params.Player == pn then
                        if (not params.LifeColor and params.Value >= 1.0) or (params.LifeColor and params.LifeColor == 'rainbow') then
                            self:diffuse(params.HSV)
                            self:diffusetopedge(0, 0.15, 0.3, 0.5)
                        end
                    end
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        self:stoptweening()
                        if params.LifeColor then
                            if params.LifeColor == 'rainbow' then
                            else
                                self:diffuse(color(params.LifeColor))
                            end
                        else
                            if params.Value < 1.0 and params.Value > 0.0 then
                                self:diffuse(params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal)
                            end
                        end
                        if params.Value >= 1.0 then
                            self:cropright(0)
                        elseif params.Value > 0.0 then
                            self:cropright(1-params.Value)
                        else
                            self:cropright(1)
                        end
                        self:diffusetopedge(0, 0.15, 0.3, 0.5)
                    end
                end,
            }),
            Def.Quad({
                OnCommand = function(self)
                    self:blend("BlendMode_Add")
                    self:zoomto(lifeWidth-2, 8)
                    self:y(-3)
                    self:diffuse(0.8, 0.8, 0.8, 1)
                    self:diffusebottomedge(0.8, 0.8, 0.8, 0.3)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        self:stoptweening()
                        if params.Value > 0.0 then
                            self:cropright(1-params.Value)
                        else
                            self:cropright(1)
                        end
                    end
                end,
            }),
            Def.Quad({
                OnCommand = function(self)
                    self:zoomto(lifeWidth-2, 18)
                    self:horizalign(left)
                    self:x(-lifeWidth/2+1)
                    self:diffuse(0, 0, 0, 0)
                    self:diffuseleftedge(0, 0, 0, 0.3)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        self:stoptweening()
                        if params.Value>=1.0 then
                            self:zoomtowidth((lifeWidth-2))
                            self:diffuseleftedge(0,0,0,0.2)
                        elseif params.Value>0.0 then
                            self:zoomtowidth((lifeWidth-2)*params.Value)
                            self:diffuseleftedge(0,0,0,0.3)
                        else
                            self:zoomtowidth(0)
                        end
                    end
                end,
            }),
        })
    end

    local function FlareIcon()
        return LoadActor(THEME:GetPathG('', 'ScreenGameplay/_parts/frame/life/icon_flare'))..{
            InitCommand = function(self)
                self:horizalign(W:IsP1(pn) and right or left)
                self:xy((260/2+WideScale(-50, 3)), WideScale(0, 8))
                self:rotationy(((pn == PLAYER_1) and 0 or 180))
                self:animate(false)
                self:setstate(0)
            end,
            CustomFrameChangedFlareCommand = function(self, params)
                if params.Player == pn then
                    if params.Level then
                        self:setstate(params.Level)
                    end
                end
            end,
        }
    end
    
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-(SCREEN_WIDTH/4-SCREEN_WIDTH/100) * ((pn == PLAYER_1) and 1 or -1), SCREEN_TOP+28)
            self:zoomx((pn == PLAYER_1) and 1 or -1)

        end,
        CustomFrameUpdateCommand = function(self, params)
            if params.Player == pn then
                local beat = 0.03*(params.Beat*10%10)
                local hue = (params.Sec or 0)*100%360
                params.HSV = HSV((hue+350)%360,1.0,1.0-(beat/2))
                self:playcommand('ScreenUpdateWithHSV', params)
            end
        end,
        LifeBar(),
        BatteryBorder(),
        FlareIcon(),
    })
end

local function ScoreActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM-24)
            self:zoom(0.95)
        end,
        Def.ActorFrame({
            OnCommand = function(self)
                self:x((-SCREEN_WIDTH/4+SCREEN_WIDTH/100) * ((pn == PLAYER_1) and 1 or -1))
                self:zoomx((pn == PLAYER_1) and 1 or -1)
            end,
            LoadActor('../'..tc..'/ScoreFrame')..{
                OnCommand = function(self)
                    self:zoomx((pn == PLAYER_1) and 1 or -1)
                    self:zoomy(1)
                    self:diffusealpha(0.8)
                end,
            },
            LoadFont('Common Square')..{
                InitCommand = function(self)
                    self:diffuse(W_PLAYER:Color(pn))
                    self:strokecolor(Color('Black'))
                    self:y(-1)
                    self:settext('')
                    self:zoomx((pn == PLAYER_1) and 0.75 or -0.75)
                    self:zoomy(0.75)
                end,
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                        local textColor = W_PLAYER:Color(pn)
                        if stats then
                            local currentScore = stats:GetScore()
                            if currentScore > displayScore then
                                displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                            elseif currentScore < displayScore then
                                displayScore = currentScore
                            end
                            local formatScore = W_STRING:Number(displayScore)
                            local lenScore = string.len(''..displayScore)
                            self:settext(formatScore)
                            self:ClearAttributes()
                            self:AddAttribute(0, {
                                Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                                Diffuse = {
                                    textColor[1]*0.5,
                                    textColor[2]*0.5,
                                    textColor[3]*0.5, 1.0},
                            })
                        end
                    end
                end,
            },
        }),
    })
end

local function DifficultyActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM-46)
        end,
        Def.ActorFrame({
            OnCommand = function(self)
                self:x((-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+50) * ((pn == PLAYER_1) and 1 or -1))
                self:zoomx((pn == PLAYER_1) and 1 or -1)
            end,
            LoadActor('LifeMeterBar Difficulty')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(bottom)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(W_DIF:Color(params.Difficulty))
                    end
                end,
            },
            LoadFont('Common Square')..{
                OnCommand = function(self)
                    self:horizalign((pn == PLAYER_1) and left or right)
                    self:vertalign(bottom)
                    self:maxwidth(90)
                    self:zoomx((pn == PLAYER_1) and 0.55 or -0.55)
                    self:zoomy(0.55)
                    self:xy(-100, -3)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(W_DIF:Color(params.Difficulty))
                        self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                        self:settextf('%s', string.upper(W_DIF:Name(params.Difficulty)))
                    end
                end,
            },
            LoadFont('Common Square')..{
                OnCommand = function(self)
                    self:horizalign((pn == PLAYER_1) and right or left)
                    self:vertalign(top)
                    self:zoomx((pn == PLAYER_1) and 0.9 or -0.9)
                    self:zoomy(0.9)
                    self:maxwidth(90)
                    self:xy(-18, -9)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(W_DIF:Color(params.Difficulty))
                        self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                        self:settextf('%d', params.Steps:GetMeter())
                    end
                end,
            },
        }),
    })
end

local function BPMActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Square')..{
            OnCommand = function(self)
                self:horizalign(center)
                if GAMESTATE:GetNumPlayersEnabled() == 1 then
                    self:x(SCREEN_CENTER_X)
                else
                    self:x(SCREEN_CENTER_X-45 * ((pn == PLAYER_1) and 1 or -1))
                end
                self:y(SCREEN_BOTTOM-25)
                self:zoom(0.6)
                self:maxwidth(120)
                self:diffuse(Color('White'))
                self:diffusebottomedge(0.75, 0.75, 0.75, 1.0)
                self:strokecolor(Color('Outline'))
            end,
            CustomFrameUpdateCommand = function(self, params)
                if params.Player == pn then
                    local steps  = W:Steps(pn)
                    local timing = steps and steps:GetTimingData() or nil
                    if timing then
                        local beat = timing:GetBeatFromElapsedTime(params.Sec)
                        local bpm  = timing:GetBPMAtBeat(beat)
                        self:settextf('%4.1f', bpm)
                    end
                end
            end,
        },
    })
end

-- フレームの端
local function FrameSideActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        local targetPn = (GAMESTATE:GetNumPlayersEnabled() > 1) and pn or GAMESTATE:GetMasterPlayerNumber()
        a[#a+1] = Def.ActorFrame({
            OnCommand = function(self)
                self:zoomx((pn == PLAYER_1) and 1 or -1)
                self:x((pn == PLAYER_1) and 0 or SCREEN_WIDTH)
            end,
            -- 上フレーム端
            LoadActor('../'..tc..'/LifeMeterBar under2')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:xy(lifeWidth-lifeWidth/4-SCREEN_WIDTH/16, SCREEN_TOP+10)
                end,
            },
            -- 下フレーム端
            LoadActor('../'..tc..'/LifeMeterBar under3')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(bottom)
                    self:xy(lifeWidth-lifeWidth/4-SCREEN_WIDTH/16, SCREEN_BOTTOM-10)
                end,
            },
            -- 発光エフェクト
            Def.ActorFrame({
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == targetPn then
                        if params.Sec >= params.LightSec then
                            self:diffusealpha(0.05*(10 - (((params.Beat+0.1)*10)%10)))
                        else
                            self:diffusealpha(0)
                        end
                    end
                end,
                LoadActor('LifeMeterBar Effect')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:horizalign(right)
                        self:vertalign(top)
                        self:xy(lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+40, SCREEN_TOP - 10)
                    end,
                },
                LoadActor('LifeMeterBar Effect')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:zoomy(-1)
                        self:horizalign(right)
                        self:vertalign(bottom)
                        self:xy(lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+40, SCREEN_HEIGHT - 110)
                    end,
                },
            }),
        })
    end
    return a
end

local function PlayerNameActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Small Bold')..{
            Text = W_PLAYER:Name(pn) or '',
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:xy((pn == PLAYER_1) and 5 or SCREEN_RIGHT-5, SCREEN_BOTTOM-7)
                self:zoom(0.5)
            end,
        },
    })
end

local function PlayerUnderlayActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        a[#a+1] = Def.ActorFrame({
            -- 難易度
            DifficultyActor(pn),
            -- スコア
            ScoreActor(pn),
            -- ライフ
            LifeActor(pn),
            -- BPM
            BPMActor(pn),
            -- フレーム端
            FrameSideActor(),
        })
    end
    return a
end

local function PlayerOverlayActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        a[#a+1] = Def.ActorFrame({
            -- プレイヤー名
            PlayerNameActor(pn),
        })
    end
    return a
end

-- フレーム背面
local function FrameUnderlayActor()
    return Def.ActorFrame({
        -- 上フレーム
        Def.Quad({
            OnCommand = function(self)
                self:vertalign(top)
                self:xy(SCREEN_CENTER_X, SCREEN_TOP)
                self:zoomto(SCREEN_WIDTH, 10)
                self:diffuse(0, 0, 0, 1)
            end,
        }),
        Def.ActorFrame({
            OnCommand = function(self)
                self:x(0)
            end,
            -- 上フレーム
            LoadActor('../'..tc..'/LifeMeterBar under1')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:xy(SCREEN_CENTER_X, SCREEN_TOP+10)
                end,
            },
            -- 上フレーム加算合成
            LoadActor('../'..tc..'/LifeMeterBar underL')..{
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:horizalign(right)
                    self:vertalign(top)
                    self:xy(SCREEN_CENTER_X, SCREEN_TOP+10)
                end,
            },
        }),
        Def.ActorFrame({
            OnCommand = function(self)
                self:zoomx(-1)
                self:x(SCREEN_WIDTH)
            end,
            -- 上フレーム
            LoadActor('../'..tc..'/LifeMeterBar under1')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:xy(SCREEN_CENTER_X, SCREEN_TOP+10)
                end,
            },
            -- 上フレーム加算合成
            LoadActor('../'..tc..'/LifeMeterBar underL')..{
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:horizalign(right)
                    self:vertalign(top)
                    self:xy(SCREEN_CENTER_X, SCREEN_TOP+10)
                end,
            },
        }),
        -- 下フレーム
        Def.ActorFrame({
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM - 55)
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:horizalign(right)
                    self:zoomto(280, 16)
                    self:diffuse(0, 0, 0, 1)
                    self:diffuseleftedge(0, 0, 0, 0)
                end,
            }),
            Def.Quad({
                OnCommand = function(self)
                    self:horizalign(left)
                    self:zoomto(280, 16)
                    self:diffuse(0, 0, 0, 1)
                    self:diffuserightedge(0, 0, 0, 0)
                end,
            }),
            LoadFont('Common Square') .. {
                OnCommand = function(self)
                    self:zoom(0.4)
                    self:maxwidth(800)
                    self:diffuse(Color('White'))
                end,
                CustomFrameChangedSongCommand = function(self, params)
                    if params.Song then
                        self:settextf('%s - %s', params.Song:GetDisplayArtist(), params.Song:GetDisplayFullTitle())
                    else
                        self:settext('')
                    end
                end,
            },
        }),
        LoadActor('../'..tc..'/LifeMeterBar bottom')..{
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
            end,
        },
        -- ステージ
        LoadFont('Common Square')..{
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, 35)
                self:zoom(0.5)
                self:diffuse(Color('White'))
                self:diffusebottomedge(0.75, 0.75, 0.75, 1.0)
                self:strokecolor(Color('Outline'))
            end,
            CustomFrameChangedSongCommand = function(self)
                self:settext(W_STAGE:String(false, true))
            end,
        },
        PlayerUnderlayActor(),
    })
end

-- フレーム前面
local function FrameOverlayActor()
    return Def.ActorFrame({
        Def.Quad({
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
                self:zoomto(SCREEN_WIDTH, 15)
                self:diffuse(0, 0, 0, 0.8)
            end,
        }),
        PlayerOverlayActor(),
    })
end

return Def.ActorFrame({
    CustomFrameOnCommand = function(self, params)
        for _,pn in pairs(PlayerNumber) do
            if params.Player == pn then
                if params.Position then params.Position:zoomy(0.89):y(26) end
                if params.Graph then params.Graph:zoomy(0.89):y(26) end
            end
        end
    end,
    FrameUnderlayActor(),
    FrameOverlayActor(),
})