local tc, tcConfig = ...
local lifeWidth = SCREEN_WIDTH/4
local topWidth = math.max(SCREEN_WIDTH-300, 400)
local bottomWidth = SCREEN_WIDTH-440

-- ライフゲージ
local function LifeActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end

    local function BatteryBorder()
        local a = Def.ActorFrame({})
        for i=1, 9 do
            a[#a+1] = LoadActor('life border')..{
                OnCommand = function(self)
                    self:visible(false)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if not params.Battery or params.Battery > 10 then
                            self:x(0)
                            self:visible(false)
                        else
                            self:x(-lifeWidth/2 + lifeWidth*i/params.Battery + 1)
                            self:visible(i < params.Battery)
                        end
                    end
                end,
            }
        end
        return a
    end

    local function LifeBar()
        return Def.ActorFrame({
            LoadActor("lifeframe_underlay")..{
                OnCommand = function(self)
                    self:animate(false);
                    self:setstate(1);
                    self:zoomtowidth(lifeWidth-2)
                end
            },
            LoadActor("lifeframe_underlay")..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(0)
                    self:horizalign(right)
                    self:x(-(lifeWidth-2)/2)
                end
            },
            LoadActor("lifeframe_underlay")..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(2)
                    self:horizalign(left)
                    self:x((lifeWidth-2)/2)
                end
            },
            LoadActor("lifeframe_left")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:x(-(lifeWidth-2)/2)
                    self:diffuse(0,0,0,0.25)
                end
            },
            LoadActor("lifeframe_left")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:rotationz(180)
                    self:x((lifeWidth-2)/2)
                    self:diffuse(0,0,0,0.25)
                end
            },
            LoadActor("lifeframe_left")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:x(-(lifeWidth-2)/2)
                    self:diffuseshift()
                    self:effectcolor1(1,1,1,0)
                    self:effectcolor2(1,1,1,0)
                    self:effectclock('bgm')
                    self:blend('BlendMode_Add')
                end,
                CustomFrameChangedLifeCommand=function(self,params)
                    if params.Player==pn then
                        local col = params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal
                        self:effectcolor1(col)
                        self:effectcolor2(col[1], col[2], col[3], 0.25)
                    end
                end,
            };
            LoadActor("lifeframe_left")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:rotationz(180)
                    self:x((lifeWidth-2)/2)
                    self:diffuseshift()
                    self:effectcolor1(1,1,1,0)
                    self:effectcolor2(1,1,1,0)
                    self:effectclock('bgm')
                    self:blend('BlendMode_Add')
                end,
                CustomFrameChangedLifeCommand=function(self,params)
                    if params.Player==pn then
                        local col = params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal
                        self:effectcolor1(col)
                        self:effectcolor2(col[1], col[2], col[3], 0.25)
                    end
                end,
            },
            LoadActor("lifeframe_border_l")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:x(-(lifeWidth-2)/2)
                end
            },
            LoadActor("lifeframe_border_l")..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:rotationz(180)
                    self:x((lifeWidth-2)/2)
                end
            },
            LoadActor("life in")..{
                OnCommand = function(self)
                    self:zoomto(lifeWidth-4,32)
                    self:blend('BlendMode_Add')
                end,
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        if params.Value>=1.0 then
                            self:cropright(0)
                        elseif params.Value > 0.0 then
                            self:cropright(1-params.Value)
                        else
                            self:cropright(1)
                        end
                        local beat2 = 0.05*(params.Beat*10%10);
                        self:croptop(0.5-beat2)
                        self:cropbottom(beat2)
                        self:y(beat2*32-8)
                    end
                end,
            },
            Def.Quad{
                OnCommand = function(self)
                    self:zoomto(lifeWidth-4, 16)
                    self:diffuse(0, 0.68, 0.93, 1)
                    self:blend('BlendMode_Add')
                    self:diffusealpha(0.5)
                end,
                ScreenUpdateWithHSVCommand = function(self, params)
                    if params.Player == pn then
                        if (not params.LifeColor and params.Value >= 1.0)
                            or (params.LifeColor and params.LifeColor == 'rainbow') then
                            self:diffuse(params.HSV)
                            self:diffusealpha(0.5)
                        end
                        if params.Value>0.0 then
                            local p = params.Value
                            self:cropright(1-p)
                        else
                            self:cropright(1)
                        end;
                    end
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if params.LifeColor then
                            if params.LifeColor == 'rainbow' then
                            else
                                self:diffuse(color(params.LifeColor))
                            end
                        elseif not params.Danger and params.Value >= 1.0 then
                            self:diffuse(tcConfig.Color.Normal)
                        elseif params.Danger then
                            self:diffuse(tcConfig.Color.Danger)
                        else
                            self:diffuse(tcConfig.Color.Normal)
                        end
                        self:diffusealpha(0.5)
                    end
                end,
            },
            Def.Quad{
                OnCommand = function(self)
                    self:zoomto(lifeWidth-4, 2)
                    self:diffuse(1, 1, 1, 0.2)
                    self:blend('BlendMode_Add')
                end,
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        local beat = (params.Beat+0.1)
                        self:y(8-16.0*(beat%1.0))
                        self:diffusealpha(0.4-(beat%1.0)*0.4)
                    end
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if params.Value > 0.0 then
                            local p = params.Value
                            self:cropright(1-p)
                        else
                            self:cropright(1)
                        end
                    end
                end,
            },
            LoadActor("lifeframe_overlay")..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(1)
                    self:zoomtowidth(lifeWidth-4)
                end,
            },
            LoadActor("lifeframe_overlay")..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(0)
                    self:horizalign(right)
                    self:x(-(lifeWidth-4)/2)
                end,
            },
            LoadActor("lifeframe_overlay")..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(2)
                    self:horizalign(left)
                    self:x((lifeWidth-4)/2)
                end,
            },
            LoadActor("light center")..{
                OnCommand = function(self)
                    self:zoomtowidth(lifeWidth*2)
                    self:blend('BlendMode_Add')
                end,
                ScreenUpdateWithHSVCommand = function(self, params)
                    if params.Player == pn then
                        if (not params.LifeColor and params.Value >= 1.0)
                            or (params.LifeColor and (params.LifeColor == 'rainbow' or params.LifeColor == '#000000ff')) then
                            self:diffuse(params.HSV)
                        end
                        local beat2 = 0.1*(params.Beat*10%20)
                        self:cropleft(0.5-(beat2/4))
                        self:cropright(beat2/4)
                        self:x((beat2/4)*(lifeWidth*2)-lifeWidth/2)
                    end
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player==pn then
                        if not params.Danger and params.Value >= 1.0 then
                            self:diffuse(tcConfig.Color.Normal)
                        elseif params.Danger then
                            self:diffuse(tcConfig.Color.Danger)
                        else
                            self:diffuse(tcConfig.Color.Normal)
                        end
                    end
                end,
            },
        })
    end

    local function FlareIcon()
        return LoadActor(THEME:GetPathG('', 'ScreenGameplay/_parts/frame/life/icon_flare'))..{
            InitCommand = function(self)
                self:horizalign(W:IsP1(pn) and right or left)
                self:xy((lifeWidth/2-1), 0.5)
                self:rotationy(((pn == PLAYER_1) and 0 or 180))
                self:animate(false)
                self:setstate(0)
            end,
            CustomFrameChangedFlareCommand = function(self, params)
                if params.Player == pn then
                    if params.Level then
                        self:setstate(params.Level)
                    end
                end
            end,
        }
    end
    
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-(SCREEN_WIDTH/4-SCREEN_WIDTH/100) * ((pn == PLAYER_1) and 1 or -1), SCREEN_TOP+27)
            self:zoomx((pn == PLAYER_1) and 1 or -1)

        end,
        CustomFrameUpdateCommand = function(self, params)
            if params.Player == pn then
                local beat = 0.03*(params.Beat*10%10)
                local hue = (params.Sec or 0)*100%360
                params.HSV = HSV((hue+350)%360,1.0,1.0-(beat/2))
                self:playcommand('ScreenUpdateWithHSV', params)
            end
        end,
        LifeBar(),
        BatteryBorder(),
        FlareIcon(),
    })
end

local function ScoreActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM-30)
        end,
        Def.ActorFrame({
            OnCommand = function(self)
                self:x((-SCREEN_WIDTH/4+SCREEN_WIDTH/100) * ((pn == PLAYER_1) and 1 or -1))
                self:zoomx((pn == PLAYER_1) and 1 or -1)
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:diffuse(Color('White'))
                    self:zoomto(160, 26)
                    self:fadeleft(0.9)
                    self:faderight(0.9)
                end,
            }),
            LoadFont('waiei2 Score Numbers')..{
                InitCommand = function(self)
                    self:diffuse(BoostColor(W_PLAYER:Color(W:APn(pn)), 1.5))
                    self:strokecolor(Color('Black'))
                    self:zoomx((pn == PLAYER_1) and 1 or -1)
                    self:y(-1)
                    self:settext('')
                end,
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                        local textColor = BoostColor(W_PLAYER:Color(W:APn(pn)), 1.5)
                        if stats then
                            local currentScore = stats:GetScore()
                            if currentScore > displayScore then
                                displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                            elseif currentScore < displayScore then
                                displayScore = currentScore
                            end
                            local formatScore = W_STRING:Number(displayScore)
                            local lenScore = string.len(''..displayScore)
                            self:settext(formatScore)
                            self:ClearAttributes()
                            self:AddAttribute(0, {
                                Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                                Diffuse = {
                                    textColor[1]*0.5,
                                    textColor[2]*0.5,
                                    textColor[3]*0.5, 1.0},
                            })
                        end
                    end
                end,
            },
        }),
    })
end

local function DifficultyActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM-46)
        end,
        Def.ActorFrame({
            OnCommand = function(self)
                self:x((-SCREEN_CENTER_X-30-WideScale(20, 0)) * ((pn == PLAYER_1) and 1 or -1))
                self:zoomx((pn == PLAYER_1) and 1 or -1)
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(top)
                    self:zoomto(180, 12)
                    self:xy(0, -9)
                    self:faderight(0.5)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(BoostColor(W_DIF:Color(params.Difficulty), 0.8))
                        self:diffusealpha(0.8)
                    end
                end,
            }),
            LoadFont('Common Small Normal')..{
                OnCommand = function(self)
                    self:horizalign((pn == PLAYER_1) and left or right)
                    self:vertalign(bottom)
                    self:maxwidth(90/0.6)
                    self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
                    self:zoomy(0.6)
                    self:xy(60, -1)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(W_DIF:Color(params.Difficulty))
                        self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                        self:settextf('%s', string.upper(W_DIF:Name(params.Difficulty)))
                    end
                end,
            },
            LoadFont('waiei2 Combo Numbers')..{
                OnCommand = function(self)
                    self:horizalign((pn == PLAYER_1) and left or right)
                    self:vertalign(top)
                    self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
                    self:zoomy(0.6)
                    self:maxwidth(90)
                    self:xy(60, 8)
                end,
                CustomFrameChangedStepsCommand = function(self, params)
                    if params.Player == pn then
                        self:diffuse(W_DIF:Color(params.Difficulty))
                        self:settextf('%d', params.Steps:GetMeter())
                    end
                end,
            },
        }),
    })
end

local function BPMActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Normal')..{
            OnCommand = function(self)
                self:horizalign(center)
                if GAMESTATE:GetNumPlayersEnabled() == 1 then
                    self:x(SCREEN_CENTER_X)
                else
                    self:x(SCREEN_CENTER_X-45 * ((pn == PLAYER_1) and 1 or -1))
                end
                self:y(SCREEN_BOTTOM-30)
                self:zoom(0.6)
                self:maxwidth(120)
                self:diffuse(Color('White'))
                self:shadowlength(1)
            end,
            CustomFrameUpdateCommand = function(self, params)
                if params.Player == pn then
                    local steps  = W:Steps(pn)
                    local timing = steps and steps:GetTimingData() or nil
                    if timing then
                        local beat = timing:GetBeatFromElapsedTime(params.Sec)
                        local bpm  = timing:GetBPMAtBeat(beat)
                        self:settextf('%4.1f', bpm)
                    end
                end
            end,
        },
    })
end

-- フレームの端
local function FrameSideActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        local targetPn = (GAMESTATE:GetNumPlayersEnabled() > 1) and pn or GAMESTATE:GetMasterPlayerNumber()
        a[#a+1] = Def.ActorFrame({
            OnCommand = function(self)
                self:zoomx((pn == PLAYER_1) and 1 or -1)
                self:x((pn == PLAYER_1) and 0 or SCREEN_WIDTH)
            end,
            -- 発光エフェクト
            Def.ActorFrame({
                CustomFrameUpdateCommand = function(self, params)
                    if params.Player == pn then
                        if params.Sec >= params.LightSec then
                            self:diffusealpha(0.05*(10 - (((params.Beat+0.1)*10)%10)))
                        else
                            self:diffusealpha(0)
                        end
                    end
                end,
                LoadActor('frame_light')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:horizalign(left)
                        self:vertalign(top)
                        self:zoomtowidth(SCREEN_CENTER_X/2)
                        self:xy(0, SCREEN_TOP)
                    end,
                },
                LoadActor('frame_light')..{
                    OnCommand = function(self)
                        self:blend('BlendMode_Add')
                        self:horizalign(left)
                        self:vertalign(top)
                        self:rotationx(180)
                        self:zoomtowidth(SCREEN_CENTER_X/2)
                        self:xy(0, SCREEN_BOTTOM)
                    end,
                },
            }),
        })
    end
    return a
end

local function PlayerNameActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Small Bold')..{
            Text = W_PLAYER:Name(pn) or '',
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:xy((pn == PLAYER_1) and 5 or SCREEN_RIGHT-5, SCREEN_BOTTOM-7)
                self:zoom(0.5)
            end,
        },
    })
end

local function PlayerUnderlayActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        a[#a+1] = Def.ActorFrame({
            -- 難易度
            DifficultyActor(pn),
            -- スコア
            ScoreActor(pn),
            -- ライフ
            LifeActor(pn),
            -- BPM
            BPMActor(pn),
            -- フレーム端
            FrameSideActor(),
        })
    end
    return a
end

local function PlayerOverlayActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        a[#a+1] = Def.ActorFrame({
            -- プレイヤー名
            PlayerNameActor(pn),
        })
    end
    return a
end

-- フレーム背面
local function FrameUnderlayActor()
    return Def.ActorFrame({
        -- 上フレーム
        LoadActor('frame_top')..{
            OnCommand = function(self)
                self:horizalign(right)
                self:vertalign(top)
                self:xy(SCREEN_CENTER_X-topWidth/2, SCREEN_TOP)
                self:animate(false)
                self:setstate(0)
            end,
        },
        LoadActor('frame_top')..{
            OnCommand = function(self)
                self:horizalign(left)
                self:vertalign(top)
                self:xy(SCREEN_CENTER_X+topWidth/2, SCREEN_TOP)
                self:animate(false)
                self:setstate(2)
            end,
        },
        LoadActor('frame_top')..{
            OnCommand = function(self)
                self:vertalign(top)
                self:xy(SCREEN_CENTER_X, SCREEN_TOP)
                self:animate(false)
                self:setstate(1)
                self:zoomtowidth(SCREEN_WIDTH - (SCREEN_CENTER_X-topWidth/2)*2)
            end,
        },
        -- 下フレーム
        Def.ActorFrame({
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM - 55)
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:horizalign(right)
                    self:zoomto(280, 16)
                    self:diffuse(0, 0, 0, 1)
                    self:diffuseleftedge(0, 0, 0, 0)
                end,
            }),
            Def.Quad({
                OnCommand = function(self)
                    self:horizalign(left)
                    self:zoomto(280, 16)
                    self:diffuse(0, 0, 0, 1)
                    self:diffuserightedge(0, 0, 0, 0)
                end,
            }),
            LoadFont('Common Normal') .. {
                OnCommand = function(self)
                    self:zoom(0.4)
                    self:maxwidth(WideScale(600, 800))
                    self:diffuse(Color('White'))
                end,
                CustomFrameChangedSongCommand = function(self, params)
                    if params.Song then
                        self:settextf('%s - %s', params.Song:GetDisplayArtist(), params.Song:GetDisplayFullTitle())
                    else
                        self:settext('')
                    end
                end,
            },
        }),
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X-bottomWidth/2, SCREEN_BOTTOM)
                self:animate(false)
                self:setstate(0)
            end,
        },
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:horizalign(left)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X+bottomWidth/2, SCREEN_BOTTOM)
                self:animate(false)
                self:setstate(2)
            end,
        },
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
                self:animate(false)
                self:setstate(1)
                self:zoomtowidth(SCREEN_WIDTH-440)
            end,
        },
        -- ステージ
        LoadFont('Common Normal')..{
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, 28)
                self:zoom(0.45)
                self:diffuse(Color('White'))
                self:diffusebottomedge(0.75, 0.75, 0.75, 1.0)
                self:strokecolor(Color('Outline'))
                self:shadowlength(1)
            end,
            CustomFrameChangedSongCommand = function(self)
                self:settext(W_STAGE:String())
            end,
        },
        PlayerUnderlayActor(),
    })
end

-- フレーム前面
local function FrameOverlayActor()
    return Def.ActorFrame({
        Def.Quad({
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
                self:zoomto(SCREEN_WIDTH, 15)
                self:diffuse(0, 0, 0, 0.8)
            end,
        }),
        PlayerOverlayActor(),
    })
end

return Def.ActorFrame({
    CustomFrameOnCommand = function(self, params)
        for _,pn in pairs(PlayerNumber) do
            if params.Player == pn then
                if params.Position then params.Position:zoomy(0.93):y(25) end
                if params.Graph then params.Graph:zoomy(0.93):y(25) end
            end
        end
    end,
    FrameUnderlayActor(),
    FrameOverlayActor(),
})