local tc, tcConfig = ...

-- ライフゲージ
local function LifeActor(pn)
    local function BatteryLife()
        local actors = Def.ActorFrame({
            CustomFrameOnCommand = function(self, params)
                if params.Player == pn then
                    self:visible(params.Battery ~= nil)
                end
            end,
            LoadActor('ya_life_battery')..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(5)
                    self:diffuse(0, 0, 0, 1)
                    self:glowshift()
                    self:effectcolor1(0, 0, 0, 1)
                    self:effectcolor2(0, 0, 0, 1)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if not params.Battery then return end
                        if params.Life:GetLivesLeft() == 1 then
                            self:effectcolor1(0, 0, 0, 1)
                            self:effectcolor2(tcConfig.Color.Danger)
                        else
                            self:effectcolor1(0, 0, 0, 1)
                            self:effectcolor2(0, 0, 0, 1)
                        end
                    end
                end,
            },
            LoadActor('ya_life_battery')..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(W_PLAYER:IsEnabled(pn) and 4 or 0)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if not params.Battery then return end
                        self:setstate(math.max(math.min(params.Life:GetLivesLeft(), 4), 0))
                    end
                end,
            },
            LoadFont('Common Square')..{
                InitCommand = function(self)
                    self:diffuse(Color('White'))
                    self:settext('')
                    self:xy(32, -2)
                    self:zoomx((pn == PLAYER_1) and 0.5 or -0.5)
                    self:zoomy(0.5)
                    self:visible(W_PLAYER:IsEnabled(pn))
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if not params.Battery then return end
                        if params.Life:GetLivesLeft() <= 0 then
                            self:settext('GAMEOVER')
                        else
                            self:settextf('%d / %d', params.Life:GetLivesLeft(), params.Life:GetTotalLives())
                        end
                    end
                end,
            },
        })
        return actors
    end
    local function NormalLife()
        local actors = Def.ActorFrame({
            CustomFrameOnCommand = function(self, params)
                if params.Player == pn then
                    self:visible(params.Battery == nil)
                end
            end,
            CustomFrameChangedLifeCommand = function(self, params)
                if params.Player == pn then
                    if params.Battery then return end
                end
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:zoomto(254, 32)
                    self:horizalign(left)
                    self:x(-127)
                    self:diffuse(tcConfig.Color.Frame)
                    self:glowshift()
                    self:effectcolor1(tcConfig.Color.Frame)
                    self:effectcolor2(tcConfig.Color.Frame)
                end,
                CustomFrameChangedLifeCommand = function(self, params)
                    if params.Player == pn then
                        if params.Danger then
                            self:effectcolor1(tcConfig.Color.Frame)
                            self:effectcolor2(tcConfig.Color.Danger)
                        else
                            self:effectcolor1(tcConfig.Color.Frame)
                            self:effectcolor2(tcConfig.Color.Frame)
                        end
                    end
                end,
            }),
        })
        if W_PLAYER:IsEnabled(pn) then
            for i=0, 19 do
                actors[#actors+1] = Def.Quad({
                    OnCommand = function(self)
                        self:zoomto(8, 32)
                        self:horizalign(left)
                        self:x(-128 + i*13)
                    end,
                    CustomFrameChangedLifeCommand = function(self, params)
                        if params.Player == pn then
                            self:stoptweening()
                            if params.LifeColor then
                                self:diffuse(params.LifeColor == 'rainbow'
                                                and HSV(params.ColorCycle, 0.75, 1)
                                                or color(params.LifeColor))
                            elseif params.Value>=1.0 then
                                self:diffuse(tcConfig.Color.Hot)
                            else
                                self:diffuse(tcConfig.Color.Normal)
                            end
                            if params.Value >= 0.05*(i+1) then
                                self:croptop(0)
                            elseif params.Value < 0.05*i then
                                self:croptop(1)
                            else
                                self:croptop(1 - (params.Value - (0.05*i))*20)
                            end
                        end
                    end,
                    CustomFrameUpdateCommand = function(self, params)
                        if params.Player == pn then
                            if params.LifeColor then
                                self:diffuse(params.LifeColor == 'rainbow'
                                                and HSV(params.ColorCycle, 0.75, 1)
                                                or color(params.LifeColor))
                            end
                        end
                    end,
                })
                actors[#actors+1] = Def.Quad({
                    OnCommand = function(self)
                        self:zoomto(8, 32)
                        self:horizalign(left)
                        self:x(-128 + i*13)
                    end,
                    CustomFrameUpdateCommand = function(self, params)
                        if params.Player == pn then
                            local white = 0.04*((params.Beat*5+(19-i))%20)
                            if params.Hot then
                                self:diffusealpha(0.6-math.abs(white-0.4)*1.5)
                            else
                                self:diffusealpha(0.8-white)
                            end
                        end
                    end,
                    CustomFrameChangedLifeCommand = function(self, params)
                        if params.Player == pn then
                            self:stoptweening()
                            if params.Value >= 0.05*(i+1) then
                                self:croptop(0)
                            elseif params.Value < 0.05*i then
                                self:croptop(1)
                            else
                                self:croptop(1 - (params.Value - (0.05*i))*20)
                            end
                        end
                    end,
                })
            end
        end
        actors[#actors+1] = LoadActor('ya_life_frame')
        -- フレア
        actors[#actors+1] = LoadActor(THEME:GetPathG('', 'ScreenGameplay/_parts/frame/life/icon_flare'))..{
            InitCommand = function(self)
                self:horizalign(W:IsP1(pn) and right or left)
                self:xy((260/2+3), 8)
                self:rotationy(((pn == PLAYER_1) and 0 or 180))
                self:animate(false)
                self:setstate(0)
            end,
            CustomFrameChangedFlareCommand = function(self, params)
                if params.Player == pn then
                    if params.Level then
                        self:setstate(params.Level)
                    end
                end
            end,
        }
        return actors
    end
    
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X - (WideScale(180, 210) * ((pn == PLAYER_1) and 1 or -1)), SCREEN_TOP+22)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
        end,
        NormalLife(),
        BatteryLife(),
    })
end

local function ScoreActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:x(SCREEN_CENTER_X-((SCREEN_WIDTH/4+SCREEN_WIDTH/100+WideScale(40, 0))*((pn == PLAYER_1) and 1 or -1)))
            self:y(SCREEN_BOTTOM-30)
        end,
        LoadActor('score_frame'),
        LoadFont('Common Square')..{
            InitCommand = function(self)
                self:diffuse((pn == PLAYER_1) and color('#80f0ffff') or color('#ffb080ff'))
                self:settext('')
                self:zoom(0.75)
                self:visible(W_PLAYER:IsEnabled(pn))
            end,
            CustomFrameUpdateCommand = function(self, params)
                if params.Player == pn then
                    local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                    if stats then
                        local currentScore = stats:GetScore()
                        if currentScore > displayScore then
                            displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                        elseif currentScore < displayScore then
                            displayScore = currentScore
                        end
                        local formatScore = W_STRING:Number(displayScore)
                        local lenScore = string.len(''..displayScore)
                        self:settext(formatScore)
                        self:ClearAttributes()
                        self:AddAttribute(0, {
                            Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                            Diffuse = color('#ffffff80'),
                        })
                    end
                end
            end,
        },
    })
end

local function DifficultyActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local difficulty
    return Def.ActorFrame({
        LoadActor('difficulty')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:xy((pn == PLAYER_1) and SCREEN_LEFT or SCREEN_RIGHT, SCREEN_BOTTOM - 42)
                self:animate(false)
            end,
            CustomFrameChangedStepsCommand = function(self, params)
                if params.Player == pn then
                    self:setstate(Enum.Reverse(Difficulty)[params.Difficulty]*2+((params.Player == PLAYER_1) and 0 or 1))
                end
            end,
        },
    })
end

local function BPMActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Normal')..{
            OnCommand = function(self)
                self:horizalign(center)
                if GAMESTATE:GetNumPlayersEnabled() == 1 then
                    self:x(SCREEN_CENTER_X)
                else
                    self:x(SCREEN_CENTER_X-((pn == PLAYER_1) and 45 or -45))
                end
                self:y(SCREEN_BOTTOM-30)
                self:zoom(0.6)
                self:maxwidth(120)
                self:diffuse(Color('White'))
                self:strokecolor(color('#00000020'))
                self:shadowlength(1)
            end,
            CustomFrameUpdateCommand = function(self, params)
                if params.Player == pn then
                    local steps  = W:Steps(pn)
                    local timing = steps and steps:GetTimingData() or nil
                    if timing then
                        local beat = timing:GetBeatFromElapsedTime(params.Sec)
                        local bpm  = timing:GetBPMAtBeat(beat)
                        self:settextf('%4.1f', bpm)
                    end
                end
            end,
        },
    })
end

local function PlayerNameActor(pn)
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    return Def.ActorFrame({
        LoadFont('Common Small Bold')..{
            Text = W_PLAYER:Name(pn) or '',
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:xy((pn == PLAYER_1) and 5 or SCREEN_RIGHT-5, SCREEN_BOTTOM-5)
                self:zoom(0.6)
                self:strokecolor(Color('Black'))
            end,
        },
    })
end

local function PlayerUnderlayActor()
    return Def.ActorFrame({})
end

local function PlayerOverlayActor()
    local a = Def.ActorFrame({})
    for _,pn in pairs(PlayerNumber) do
        a[#a+1] = Def.ActorFrame({
            -- 難易度
            DifficultyActor(pn),
            -- スコア
            ScoreActor(pn),
            -- ライフ
            LifeActor(pn),
            -- BPM
            BPMActor(pn),
            -- プレイヤー名
            PlayerNameActor(pn),
        })
    end
    return a
end

-- フレーム背面
local function FrameUnderlayActor()
    return Def.ActorFrame({
        PlayerUnderlayActor(),
    })
end

-- フレーム前面
local function FrameOverlayActor()
    return Def.ActorFrame({
        -- 上フレーム
        Def.ActorFrame({
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, SCREEN_TOP-10)
            end,
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:animate(false)
                    self:setstate(2)
                end,
            },
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(top)
                    self:x(-SCREEN_CENTER_X)
                    self:animate(false)
                    self:setstate(0)
                end,
            },
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(top)
                    self:x(-SCREEN_CENTER_X+80)
                    self:animate(false)
                    self:setstate(1)
                    self:zoomtowidth(SCREEN_WIDTH/2-160)
                end,
            },
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:zoomx(-1)
                    self:animate(false)
                    self:setstate(2)
                end,
            },
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(left)
                    self:vertalign(top)
                    self:x(SCREEN_CENTER_X)
                    self:zoomx(-1)
                    self:animate(false)
                    self:setstate(0)
                end,
            },
            LoadActor('frame_top')..{
                OnCommand = function(self)
                    self:horizalign(right)
                    self:vertalign(top)
                    self:x(SCREEN_CENTER_X-80)
                    self:zoomx(-1)
                    self:animate(false)
                    self:setstate(1)
                    self:zoomtowidth(SCREEN_WIDTH/2-160)
                end,
            },
        }),
        -- 下フレーム
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:horizalign(left)
                self:vertalign(bottom)
                self:xy(SCREEN_LEFT, SCREEN_BOTTOM)
                self:animate(false)
                self:setstate(0)
            end,
        },
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:horizalign(left)
                self:vertalign(bottom)
                self:xy(SCREEN_RIGHT, SCREEN_BOTTOM)
                self:zoomx(-1)
                self:animate(false)
                self:setstate(0)
            end,
        },
        LoadActor('frame_bottom')..{
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
                self:animate(false)
                self:setstate(1)
                self:zoomtowidth(SCREEN_WIDTH-120)
            end,
        },
        -- ステージ
        LoadActor('stage_icon')..{
            OnCommand = function(self)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X, SCREEN_TOP-10)
                self:vertalign(top)
            end,
        },
        LoadFont('Common Bold')..{
            OnCommand = function(self)
                self:xy(SCREEN_CENTER_X, 18)
                self:zoom(0.5)
                self:diffuse(Color('White'))
                self:diffusebottomedge(0.75, 0.75, 0.75, 1.0)
                self:strokecolor(Color('Black'))
                self:diffuseshift()
                self:effectclock('bgm')
                self:effectcolor1(1,1,1,1)
                self:effectcolor2(0.6,0.6,0.6,1)
            end,
            ChangedGameplaySongMessageCommand = function(self)
                self:settext(W_STAGE:String(false, true))
            end,
        },
        PlayerOverlayActor(),
    })
end

return Def.ActorFrame({
    CustomFrameOnCommand = function(self, params)
        for _,pn in pairs(PlayerNumber) do
            if params.Player == pn then
                if params.Position then params.Position:zoomy(0.92):y(14) end
                if params.Graph then params.Graph:zoomy(0.92):y(14) end
            end
        end
    end,
    FrameUnderlayActor(),
    FrameOverlayActor(),
})