local pn, tc, tcConfig = ...
local screen
local life
local battery
local lifeWidth = SCREEN_WIDTH/4
local frameLight = W_OPTION:ReadPref('FrameLight', 'Auto')
local lightSec = 999999

-- ライフゲージ
local function LifeActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    
    local function BatteryBorder()
        local actors = Def.ActorFrame({})
        for i=1, 9 do
            actors[#actors+1] = LoadActor('LifeMeterBar frame Border')..{
                OnCommand = function(self)
                    self:visible(false)
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not battery or battery > 10 then
                        self:x(0)
                        self:visible(false)
                    else
                        self:x(-lifeWidth/2 + lifeWidth*i/battery + 1)
                        self:visible(i<battery)
                    end
                end,
            }
        end
        return actors
    end
    
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-SCREEN_WIDTH/4+SCREEN_WIDTH/100, SCREEN_TOP+18)
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local beat = 0.03*(params.PnBeat[pn]*10%10)
            local hue = (params.Sec or 0)*100%360
            params.HSV = HSV((hue+350)%360,1.0,1.0-(beat/2))
            self:playcommand('ScreenUpdateWithHSV', params)
        end,
        LoadActor('LifeMeterBar frame left')..{
            OnCommand = function(self)
                self:x(-lifeWidth/2 - 12)
            end,
        },
        LoadActor('LifeMeterBar frame right')..{
            OnCommand = function(self)
                self:x(lifeWidth/2 + 12)
            end,
        },
        LoadActor('LifeMeterBar frame Main')..{
            OnCommand = function(self)
                self:zoomtowidth(lifeWidth)
            end,
        },
        Def.ActorFrame({
            ScreenUpdateMessageCommand = function(self, params)
                self:diffusealpha(0.1*(10 - (((params.PnBeat[pn]+0.1)*10)%10)))
            end,
            LoadActor('LifeMeterBar light left')..{
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:diffuse(tcConfig.Color.Frame)
                    self:x(-lifeWidth/2 - 12)
                end,
            },
            LoadActor('LifeMeterBar light right')..{
                OnCommand = function(self)
                    self:blend('BlendMode_Add')
                    self:diffuse(tcConfig.Color.Frame)
                    self:x(lifeWidth/2 + 12)
                end,
            },
        }),
        LoadActor('LifeMeterBar scrlight')..{
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:diffuse(tcConfig.Color.Normal)
                self:zoomtowidth(lifeWidth*2)
                self:diffuse(tcConfig.Color.Normal)
            end,
            ScreenUpdateWithHSVCommand = function(self, params)
                local beat2 = 0.1*(params.PnBeat[pn]*10%20)
                self:cropleft(0.5-(beat2/4))
                self:cropright(beat2/4)
                self:x((beat2/4)*(lifeWidth*2)-lifeWidth/2)
                local curLife = life and life:GetLife() or 0
                if curLife >= 1.0 then
                    self:diffuse(params.HSV)
                end
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    local col = {}
                    if battery and life:GetTotalLives() == 1 then
                        col = tcConfig.Color.Hot
                    elseif params.Life >= 1 then
                        col = tcConfig.Color.Normal
                    elseif params.Danger then
                        col = tcConfig.Color.Danger
                    else
                        col = tcConfig.Color.Normal
                    end
                    self:diffuse(col);
                end
            end,
        },
        Def.Quad({
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:diffuse(tcConfig.Color.Danger)
                self:zoomto(lifeWidth-2, 18)
            end,
            ScreenUpdateWithHSVCommand = function(self, params)
                local curLife = life and life:GetLife() or 0
                if curLife >= 1.0 then
                    self:diffuse(params.HSV)
                    self:diffusetopedge(0, 0.15, 0.3, 0.5)
                end
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    self:linear(0.1)
                    if params.Life>=1.0 then
                        self:cropright(0)
                    elseif params.Life > 0.0 then
                        self:diffuse(params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal)
                        self:cropright(1-params.Life)
                    else
                        self:cropright(1)
                    end
                    self:diffusetopedge(0, 0.15, 0.3, 0.5)
                end
            end,
        }),
        Def.Quad({
            OnCommand = function(self)
                self:blend("BlendMode_Add")
                self:zoomto(lifeWidth-2, 8)
                self:y(-3)
                self:diffuse(0.8, 0.8, 0.8, 1)
                self:diffusebottomedge(0.2, 0.2, 0.2, 0)
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    self:linear(0.1)
                    if params.Life > 0.0 then
                        self:cropright(1-params.Life)
                    else
                        self:cropright(1)
                    end
                end
            end,
        }),
        Def.Quad({
            OnCommand = function(self)
                self:zoomto(lifeWidth-2, 18)
                self:horizalign(left)
                self:x(-lifeWidth/2+1)
                self:diffuse(0, 0, 0, 0)
                self:diffuseleftedge(0, 0, 0, 0.3)
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    self:linear(0.1)
                    if params.Life>=1.0 then
                        self:zoomtowidth((lifeWidth-2))
                        self:diffuseleftedge(0,0,0,0.2)
                    elseif params.Life>0.0 then
                        self:zoomtowidth((lifeWidth-2)*params.Life)
                        self:diffuseleftedge(0,0,0,0.3)
                    else
                        self:zoomtowidth(0)
                    end
                end
            end,
        }),
        BatteryBorder(),
    })
end

local function ScoreActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
            self:xy(SCREEN_CENTER_X-SCREEN_WIDTH/4+SCREEN_WIDTH/100, SCREEN_BOTTOM-25)
        end,
        LoadActor('../'..tc..'/ScoreFrame')..{
            OnCommand = function(self)
                self:zoomy(0.7)
                self:diffusealpha(0.8)
            end,
        },
        LoadFont('waiei2 Score Numbers')..{
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:y(-1)
                self:settext('')
            end,
            ScreenUpdateMessageCommand = function(self, params)
                local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                if stats then
                    local currentScore = stats:GetScore()
                    if currentScore > displayScore then
                        displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                    elseif currentScore < displayScore then
                        displayScore = currentScore
                    end
                    local formatScore = W_STRING:Number(displayScore)
                    local lenScore = string.len(''..displayScore)
                    self:settext(formatScore)
                    self:ClearAttributes()
                    self:AddAttribute(0, {
                        Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                        Diffuse = color('#ffffff80'),
                    })
                end
            end,
        },
    })
end

local function DifficultyActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local difficulty
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+50, SCREEN_BOTTOM-46)
        end,
        ChangedGameplaySongMessageCommand = function(self, params)
            local steps = W:Steps(pn)
            difficulty = steps:GetDifficulty()
            self:playcommand('ChangedPlayerDifficulty', {
                Player     = pn,
                Steps      = steps,
                Difficulty = difficulty,
            })
        end,
        LoadActor('LifeMeterBar Difficulty')..{
            OnCommand = function(self)
                self:horizalign(right)
                self:vertalign(bottom)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(W_DIF:Color(params.Difficulty))
                end
            end,
        },
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:maxwidth(90)
                self:zoomx((pn == PLAYER_1) and 0.55 or -0.55)
                self:zoomy(0.55)
                self:xy(-100, -3)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(W_DIF:Color(params.Difficulty))
                    self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                    self:settextf('%s', string.upper(W_DIF:Name(params.Difficulty)))
                end
            end,
        },
        LoadFont('Common Bold')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and right or left)
                self:vertalign(top)
                self:zoomx((pn == PLAYER_1) and 0.9 or -0.9)
                self:zoomy(0.9)
                self:maxwidth(90)
                self:xy(-18, -9)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(W_DIF:Color(params.Difficulty))
                    self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                    self:settextf('%d', params.Steps:GetMeter())
                end
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        if W_PLAYER:IsEnabled(pn) then
            screen  = SCREENMAN:GetTopScreen()
            life    = screen:GetLifeMeter(pn) or nil
            battery = life.GetTotalLives and life:GetTotalLives() or nil
        end
    end,
    AfterOnCommand = function(self)
        local position = screen:GetChild('Overlay'):GetChild('Overlay'):GetChild('Position'):GetChild(pn)
        local graph = screen:GetChild('Underlay'):GetChild('Underlay'):GetChild('Note Graph'):GetChild(pn)
        if position then position:zoomy(0.92):y(14) end
        if graph then graph:zoomy(0.92):y(14) end
    end,
    ChangedGameplaySongMessageCommand = function(self, params)
        if frameLight == 'Never' then
            lightSec = 999999
        elseif frameLight == 'Always' then
            lightSec = 0
        else
            lightSec = params.Song and params.Song:GetSampleStart() or 999999
        end

    end,
    -- 上フレーム
    LoadActor('../'..tc..'/LifeMeterBar under1')..{
        OnCommand = function(self)
            self:horizalign(right)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X, SCREEN_TOP)
        end,
    },
    -- 上フレーム加算合成
    LoadActor('../'..tc..'/LifeMeterBar underL')..{
        OnCommand = function(self)
            self:blend('BlendMode_Add')
            self:horizalign(right)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X, SCREEN_TOP)
        end,
    },
    -- 下フレーム
    LoadActor('../'..tc..'/LifeMeterBar bottom')..{
        OnCommand = function(self)
            self:vertalign(bottom)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
            if pn == PLAYER_1 then
                self:cropright(0.5)
            else
                self:zoomx(-1)
                self:cropleft(0.5)
            end
        end,
    },
    -- 難易度
    DifficultyActor(),
    -- スコア
    ScoreActor(),
    -- ライフ
    LifeActor(),
    -- BPM
    LoadFont('Common Normal')..{
        OnCommand = function(self)
            self:horizalign(center)
            if GAMESTATE:GetNumPlayersEnabled() == 1 then
                self:x(SCREEN_CENTER_X)
            else
                self:x(SCREEN_CENTER_X-45)
            end
            self:y(SCREEN_BOTTOM-25)
            self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
            self:zoomy(0.6)
            self:maxwidth(120)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local steps  = W:Steps(pn)
            local timing = steps and steps:GetTimingData() or nil
            if timing then
                local beat = timing:GetBeatFromElapsedTime(params.Sec)
                local bpm  = timing:GetBPMAtBeat(beat)
                self:settextf('%4.1f', bpm)
            end
        end,
    },
    -- 上フレーム端
    LoadActor('../'..tc..'/LifeMeterBar under2')..{
        OnCommand = function(self)
            self:horizalign(right)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16, SCREEN_TOP)
        end,
    },
    -- 下フレーム端
    LoadActor('../'..tc..'/LifeMeterBar under3')..{
        OnCommand = function(self)
            self:horizalign(right)
            self:vertalign(bottom)
            self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16, SCREEN_BOTTOM-10)
        end,
    },
    Def.Quad({
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and left or right)
            self:vertalign(bottom)
            self:y(SCREEN_BOTTOM)
            self:zoomto((pn == PLAYER_1) and SCREEN_CENTER_X or -SCREEN_CENTER_X, 15)
            self:diffuse(0, 0, 0, 0.8)
        end,
    }),
    -- 発光エフェクト
    Def.ActorFrame({
        ScreenUpdateMessageCommand = function(self, params)
            if params.Sec >= lightSec then
                self:diffusealpha(0.05*(10 - (((params.PnBeat[pn]+0.1)*10)%10)))
            else
                self:diffusealpha(0)
            end
        end,
        LoadActor('LifeMeterBar Effect')..{
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:horizalign(right)
                self:vertalign(top)
                self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+40, SCREEN_TOP - 20)
            end,
        },
        LoadActor('LifeMeterBar Effect')..{
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:zoomy(-1)
                self:horizalign(right)
                self:vertalign(bottom)
                self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+40, SCREEN_HEIGHT - 110)
            end,
        },
    }),
    -- プレイヤー名
    LoadFont('Common Small Bold')..{
        Text = W_PLAYER:IsEnabled(pn) and W_PLAYER:Name(pn) or '',
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and left or right)
            self:vertalign(bottom)
            self:xy(5, SCREEN_BOTTOM-7)
            self:zoomx((pn == PLAYER_1) and 0.5 or -0.5)
            self:zoomy(0.5)
        end,
    },
})