local pn, tc, tcConfig = ...
local screen
local life
local battery
local lifeWidth = SCREEN_WIDTH/4 + 50
local frameLight = W_OPTION:ReadPref('FrameLight', 'Auto')
local lightSec = 999999

-- ライフゲージ
local function LifeActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    
    local function BatteryBorder()
        local actors = Def.ActorFrame({})
        for i=1, 9 do
            actors[#actors+1] = LoadActor('life border')..{
                OnCommand = function(self)
                    self:visible(false)
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not battery or battery > 10 then
                        self:x(0)
                        self:visible(false)
                    else
                        self:x(-lifeWidth/2 + lifeWidth*i/battery + 1)
                        self:visible(i<battery)
                    end
                end,
            }
        end
        return actors
    end
    
    local tmpLife = 0
    local oldLife = 0
    local isDanger = false
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-SCREEN_WIDTH/4+SCREEN_WIDTH/100, SCREEN_TOP+27)
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local beat = 0.03*(params.PnBeat[pn]*10%10)
            local hue = (params.Sec or 0)*100%360
            if tmpLife ~= oldLife then
                if math.abs(tmpLife - oldLife) < 0.0005 then
                    oldLife = tmpLife
                else
                    oldLife = oldLife + (tmpLife-oldLife)/4
                end
            end
            params.HSV = HSV((hue+350)%360,1.0,1.0-(beat/2))
            params.Beat = params.PnBeat[pn]
            params.Life = oldLife
            params.Player = pn
            self:playcommand('PlayerLife', params)
        end,
        ChangedPlayerLifeMessageCommand = function(self, params)
            if params.Player == pn then
                tmpLife = params.Life
                isDanger = (battery and battery <= 1) or params.Danger
            end
        end,
        LoadActor("lifeframe_underlay")..{
            OnCommand = function(self)
                self:animate(false);
                self:setstate(1);
                self:zoomtowidth(lifeWidth-2)
            end
        },
        LoadActor("lifeframe_underlay")..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(0)
                self:horizalign(right)
                self:x(-(lifeWidth-2)/2)
            end
        },
        LoadActor("lifeframe_underlay")..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(2)
                self:horizalign(left)
                self:x((lifeWidth-2)/2)
            end
        },
        LoadActor("lifeframe_left")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:x(-(lifeWidth-2)/2)
                self:diffuse(0,0,0,0.25)
            end
        },
        LoadActor("lifeframe_left")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:rotationz(180)
                self:x((lifeWidth-2)/2)
                self:diffuse(0,0,0,0.25)
            end
        },
        LoadActor("lifeframe_left")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:x(-(lifeWidth-2)/2)
                self:diffuseshift()
                self:effectcolor1(1,1,1,0)
                self:effectcolor2(1,1,1,0)
                self:effectclock('bgm')
                self:blend('BlendMode_Add')
            end,
            ChangedPlayerLifeMessageCommand=function(self,params)
                if params.Player==pn then
                    local col = params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal
                    self:effectcolor1(col)
                    self:effectcolor2(col[1], col[2], col[3], 0.25)
                end
            end,
        };
        LoadActor("lifeframe_left")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:rotationz(180)
                self:x((lifeWidth-2)/2)
                self:diffuseshift()
                self:effectcolor1(1,1,1,0)
                self:effectcolor2(1,1,1,0)
                self:effectclock('bgm')
                self:blend('BlendMode_Add')
            end,
            ChangedPlayerLifeMessageCommand=function(self,params)
                if params.Player==pn then
                    local col = params.Danger and tcConfig.Color.Danger or tcConfig.Color.Normal
                    self:effectcolor1(col)
                    self:effectcolor2(col[1], col[2], col[3], 0.25)
                end
            end,
        },
        LoadActor("lifeframe_border_l")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:x(-(lifeWidth-2)/2)
            end
        },
        LoadActor("lifeframe_border_l")..{
            OnCommand = function(self)
                self:horizalign(right)
                self:rotationz(180)
                self:x((lifeWidth-2)/2)
            end
        },
        LoadActor("life in")..{
            OnCommand = function(self)
                self:zoomto(lifeWidth-4,32)
                self:blend('BlendMode_Add')
            end,
            PlayerLifeCommand = function(self, params)
                if params.Player == pn then
                    if params.Life>=1.0 then
                        self:cropright(0)
                    elseif params.Life > 0.0 then
                        self:cropright(1-params.Life)
                    else
                        self:cropright(1)
                    end
                    local beat2 = 0.05*(params.Beat*10%10);
                    self:croptop(0.5-beat2)
                    self:cropbottom(beat2)
                    self:y(beat2*32-8)
                end
            end,
        },
        Def.Quad{
            OnCommand = function(self)
                self:zoomto(lifeWidth-4, 16)
                self:diffuse(0, 0.68, 0.93, 1)
                self:blend('BlendMode_Add')
            end,
            PlayerLifeCommand = function(self, params)
                if params.Player == pn then
                    if not isDanger and params.Life >= 1.0 then
                        self:diffuse(params.HSV)
                    elseif isDanger then
                        self:diffuse(tcConfig.Color.Danger)
                    else
                        self:diffuse(tcConfig.Color.Normal)
                    end
                    if params.Life>0.0 then
                        local p = params.Life
                        self:cropright(1-p)
                    else
                        self:cropright(1)
                    end;
                    self:diffusealpha(0.5)
                    self:blend('BlendMode_Add')
                end
            end,
        },
        Def.Quad{
            OnCommand = function(self)
                self:zoomto(lifeWidth-4, 2)
                self:diffuse(1, 1, 1, 0.2)
                self:blend('BlendMode_Add')
            end,
            PlayerLifeCommand = function(self, params)
                if params.Player == pn then
                    if params.Life > 0.0 then
                        local p = params.Life
                        self:cropright(1-p)
                    else
                        self:cropright(1)
                    end
                    local beat_ = (params.Beat+0.1)
                    self:y(8-16.0*(beat_%1.0))
                    self:diffusealpha(0.4-(beat_%1.0)*0.4)
                end
            end,
        },
        LoadActor("lifeframe_overlay")..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(1)
                self:zoomtowidth(lifeWidth-4)
            end,
        },
        LoadActor("lifeframe_overlay")..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(0)
                self:horizalign(right)
                self:x(-(lifeWidth-4)/2)
            end,
        },
        LoadActor("lifeframe_overlay")..{
            OnCommand = function(self)
                self:animate(false)
                self:setstate(2)
                self:horizalign(left)
                self:x((lifeWidth-4)/2)
            end,
        },
        LoadActor("light center")..{
            OnCommand = function(self)
                self:zoomtowidth(lifeWidth*2)
                self:blend('BlendMode_Add')
            end,
            PlayerLifeCommand = function(self, params)
                if params.Player==pn then
                    if not isDanger and params.Life >= 1.0 then
                        self:diffuse(params.HSV)
                    elseif isDanger then
                        self:diffuse(tcConfig.Color.Danger)
                    else
                        self:diffuse(tcConfig.Color.Normal)
                    end
                    local beat2 = 0.1*(params.Beat*10%20)
                    self:cropleft(0.5-(beat2/4))
                    self:cropright(beat2/4)
                    self:x((beat2/4)*(lifeWidth*2)-lifeWidth/2)
                end
            end,
        },
        BatteryBorder(),
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:diffuse(1, 1, 1, 1)
                self:strokecolor(Color('Outline'))
                self:skewx(-0.2)
                --self:horizalign(W:IsP1(pn) and right or left)
                self:horizalign(left)
                self:xy(lifeWidth/2-((pn == PLAYER_1) and 70 or 10), 0)
                self:zoom(0.45)
                self:rotationy(((pn == PLAYER_1) and 0 or 180))
                self:settext('')
            end,
            ChangedLifeTypeCommand = function(self, params)
                if not params.Level then return end
                if params.Player == pn then
                    lifeColor = params.Color
                    self:settext(params.Name or '')
                end
            end,
        },
    })
end

local function ScoreActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
            self:xy(SCREEN_CENTER_X-SCREEN_WIDTH/4+SCREEN_WIDTH/100, SCREEN_BOTTOM-30)
        end,
        Def.Quad({
            OnCommand = function(self)
                self:diffuse(Color('White'))
                self:zoomto(160, 26)
                self:fadeleft(0.9)
                self:faderight(0.9)
            end,
        }),
        LoadFont('waiei2 Score Numbers')..{
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:y(-1)
                self:settext('')
            end,
            ScreenUpdateMessageCommand = function(self, params)
                local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                if stats then
                    local currentScore = stats:GetScore()
                    if currentScore > displayScore then
                        displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                    elseif currentScore < displayScore then
                        displayScore = currentScore
                    end
                    local formatScore = W_STRING:Number(displayScore)
                    local lenScore = string.len(''..displayScore)
                    self:settext(formatScore)
                    self:ClearAttributes()
                    self:AddAttribute(0, {
                        Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                        Diffuse = color('#ffffff80'),
                    })
                end
            end,
        },
    })
end

local function DifficultyActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local difficulty
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X-lifeWidth-lifeWidth/4-SCREEN_WIDTH/16+120, SCREEN_BOTTOM-46)
        end,
        ChangedGameplaySongMessageCommand = function(self, params)
            local steps = W:Steps(pn)
            difficulty = steps:GetDifficulty()
            self:playcommand('ChangedPlayerDifficulty', {
                Player     = pn,
                Steps      = steps,
                Difficulty = difficulty,
            })
        end,
        Def.Quad({
            OnCommand = function(self)
                self:horizalign(right)
                self:vertalign(top)
                self:zoomto(180, 12)
                self:y(-9)
                self:faderight(0.5)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(BoostColor(W_DIF:Color(params.Difficulty), 0.8))
                    self:diffusealpha(0.8)
                end
            end,
        }),
        LoadFont('Common Small Normal')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:maxwidth(90/0.6)
                self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
                self:zoomy(0.6)
                self:xy(-120, -1)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(W_DIF:Color(params.Difficulty))
                    self:strokecolor(BoostColor(W_DIF:Color(params.Difficulty), 0.35))
                    self:settextf('%s', string.upper(W_DIF:Name(params.Difficulty)))
                end
            end,
        },
        LoadFont('waiei2 Combo Numbers')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(top)
                self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
                self:zoomy(0.6)
                self:maxwidth(90)
                self:xy(-120, 8)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:diffuse(W_DIF:Color(params.Difficulty))
                    self:settextf('%d', params.Steps:GetMeter())
                end
            end,
        },
    })
end

local topWidth = math.max(SCREEN_WIDTH-300, 400)
local bottomWidth = SCREEN_WIDTH-440
return Def.ActorFrame({
    OnCommand = function(self)
        if W_PLAYER:IsEnabled(pn) then
            screen  = SCREENMAN:GetTopScreen()
            life    = screen:GetLifeMeter(pn) or nil
            battery = life.GetTotalLives and life:GetTotalLives() or nil
        end
    end,
    AfterOnCommand = function(self)
        local position = screen:GetChild('Overlay'):GetChild('Overlay'):GetChild('Position'):GetChild(pn)
        local graph = screen:GetChild('Underlay'):GetChild('Underlay'):GetChild('Note Graph'):GetChild(pn)
        if position then position:zoomy(0.93):y(25) end
        if graph then graph:zoomy(0.93):y(25) end
    end,
    ChangedGameplaySongMessageCommand = function(self, params)
        if frameLight == 'Never' then
            lightSec = 999999
        elseif frameLight == 'Always' then
            lightSec = 0
        else
            lightSec = params.Song and params.Song:GetSampleStart() or 999999
        end

    end,
    -- 上フレーム
    LoadActor('frame_top')..{
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and right or left)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X-topWidth/2, SCREEN_TOP)
            self:animate(false)
            self:setstate((pn==PLAYER_1) and 0 or 2)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
        end,
    },
    -- 上フレーム
    LoadActor('frame_top')..{
        OnCommand = function(self)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X, SCREEN_TOP)
            self:animate(false)
            self:setstate(1)
            if pn == PLAYER_1 then
                self:zoomtowidth(SCREEN_WIDTH - (SCREEN_CENTER_X-topWidth/2)*2)
                self:cropright(0.5)
            else
                self:zoomtowidth(-1*(SCREEN_WIDTH - (SCREEN_CENTER_X-topWidth/2)*2))
                self:cropleft(0.5)
            end
        end,
    },
    -- 下フレーム
    LoadActor('frame_bottom')..{
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and right or left)
            self:vertalign(bottom)
            self:xy(SCREEN_CENTER_X-bottomWidth/2, SCREEN_BOTTOM)
            self:animate(false)
            self:setstate((pn==PLAYER_1) and 0 or 2)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
        end,
    },
    -- 下フレーム
    LoadActor('frame_bottom')..{
        OnCommand = function(self)
            self:vertalign(bottom)
            self:xy(SCREEN_CENTER_X, SCREEN_BOTTOM)
            self:animate(false)
            self:setstate(1)
            if pn == PLAYER_1 then
                self:zoomtowidth(SCREEN_WIDTH-440)
                self:cropright(0.5)
            else
                self:zoomtowidth(-1*(SCREEN_WIDTH-440))
                self:cropleft(0.5)
            end
        end,
    },
    -- 発光エフェクト
    Def.ActorFrame({
        ScreenUpdateMessageCommand = function(self, params)
            if params.Sec >= lightSec then
                self:diffusealpha(0.05*(10 - (((params.PnBeat[pn]+0.1)*10)%10)))
            else
                self:diffusealpha(0)
            end
        end,
        LoadActor('frame_light')..{
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:horizalign(left)
                self:vertalign(top)
                self:zoomtowidth(SCREEN_CENTER_X/2)
                self:xy(0, SCREEN_TOP)
            end,
        },
        LoadActor('frame_light')..{
            OnCommand = function(self)
                self:blend('BlendMode_Add')
                self:horizalign(left)
                self:vertalign(top)
                self:rotationx(180)
                self:zoomtowidth(SCREEN_CENTER_X/2)
                self:xy(0, SCREEN_BOTTOM)
            end,
        },
    }),
    -- 難易度
    DifficultyActor(),
    -- スコア
    ScoreActor(),
    -- -- ライフ
    LifeActor(),
    -- BPM
    LoadFont('Common Normal')..{
        OnCommand = function(self)
            self:horizalign(center)
            if GAMESTATE:GetNumPlayersEnabled() == 1 then
                self:x(SCREEN_CENTER_X)
            else
                self:x(SCREEN_CENTER_X-45)
            end
            self:y(SCREEN_BOTTOM-30)
            self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
            self:zoomy(0.6)
            self:maxwidth(120)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local steps  = W:Steps(pn)
            local timing = steps and steps:GetTimingData() or nil
            if timing then
                local beat = timing:GetBeatFromElapsedTime(params.Sec)
                local bpm  = timing:GetBPMAtBeat(beat)
                self:settextf('%4.1f', bpm)
            end
        end,
    },
    Def.Quad({
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and left or right)
            self:vertalign(bottom)
            self:y(SCREEN_BOTTOM)
            self:zoomto((pn == PLAYER_1) and SCREEN_CENTER_X or -SCREEN_CENTER_X, 15)
            self:diffuse(0, 0, 0, 0.8)
        end,
    }),
    -- プレイヤー名
    LoadFont('Common Small Bold')..{
        Text = W_PLAYER:IsEnabled(pn) and W_PLAYER:Name(pn) or '',
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and left or right)
            self:vertalign(bottom)
            self:xy(5, SCREEN_BOTTOM-7)
            self:zoomx((pn == PLAYER_1) and 0.5 or -0.5)
            self:zoomy(0.5)
        end,
    },
})