local pn, tc, tcConfig = ...
local screen
local life
local battery

-- ライフゲージ
local function LifeActor()    
    local function BatteryLife()
        local actors = Def.ActorFrame({
            AfterOnCommand = function(self)
                self:visible(battery ~= nil)
            end,
            LoadActor('ya_life_battery')..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(5)
                    self:diffuse(0, 0, 0, 1)
                    self:glowshift()
                    self:effectcolor1(0, 0, 0, 1)
                    self:effectcolor2(0, 0, 0, 1)
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not battery then return end
                    if params.Player == pn then
                        if life:GetLivesLeft() == 1 then
                            self:effectcolor1(0, 0, 0, 1)
                            self:effectcolor2(tcConfig.Color.Danger)
                        else
                            self:effectcolor1(0, 0, 0, 1)
                            self:effectcolor2(0, 0, 0, 1)
                        end
                    end
                end,
            },
            LoadActor('ya_life_battery')..{
                OnCommand = function(self)
                    self:animate(false)
                    self:setstate(W_PLAYER:IsEnabled(pn) and 4 or 0)
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not battery then return end
                    if params.Player == pn then
                        self:setstate(math.max(math.min(life:GetLivesLeft(), 4), 0))
                    end
                end,
            },
            LoadFont('Common Small Bold')..{
                InitCommand = function(self)
                    self:diffuse(Color('White'))
                    self:settext('')
                    self:xy(32, -2)
                    self:zoomx((pn == PLAYER_1) and 0.5 or -0.5)
                    self:zoomy(0.5)
                    self:visible(W_PLAYER:IsEnabled(pn))
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not battery or not W_PLAYER:IsEnabled(pn) then return end
                    if life:GetLivesLeft() <= 0 then
                        self:settext('GAMEOVER')
                    else
                        self:settextf('%d / %d', life:GetLivesLeft(), life:GetTotalLives())
                    end
                end,
            },
        })
        return actors
    end
    local pnLifeMax = false
    local tmpLife = 0
    local oldLife = 0
    local lifeColor = nil
    local colorCircle = 0
    local function NormalLife()
        local actors = Def.ActorFrame({
            AfterOnCommand = function(self)
                self:visible(battery == nil)
            end,
            ScreenUpdateMessageCommand = function(self, params)
                if battery then return end
                colorCircle = math.floor(params.Sec*100)
                if tmpLife ~= oldLife then
                    if math.abs(tmpLife - oldLife) < 0.0005 then
                        oldLife = tmpLife
                    else
                        oldLife = oldLife + (tmpLife-oldLife)/4
                    end
                    self:playcommand('PlayerLife', {
                        Player = pn,
                        Life = oldLife,
                    })
                end
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if battery then return end
                if params.Player == pn then
                    tmpLife = params.Life
                end
            end,
            Def.Quad({
                OnCommand = function(self)
                    self:zoomto(254, 32)
                    self:horizalign(left)
                    self:x(-127)
                    self:diffuse(tcConfig.Color.Frame)
                    self:glowshift()
                    self:effectcolor1(tcConfig.Color.Frame)
                    self:effectcolor2(tcConfig.Color.Frame)
                end,
                ChangedPlayerLifeMessageCommand = function(self, params)
                    if not W_PLAYER:IsEnabled(pn) then return end
                    if params.Player == pn then
                        if params.Danger then
                            self:effectcolor1(tcConfig.Color.Frame)
                            self:effectcolor2(tcConfig.Color.Danger)
                        else
                            self:effectcolor1(tcConfig.Color.Frame)
                            self:effectcolor2(tcConfig.Color.Frame)
                        end
                    end
                end,
            }),
        })
        if W_PLAYER:IsEnabled(pn) then
            for i=0, 19 do
                actors[#actors+1] = Def.Quad({
                    OnCommand = function(self)
                        self:zoomto(8, 32)
                        self:horizalign(left)
                        self:x(-128 + i*13)
                    end,
                    PlayerLifeCommand = function(self, params)
                        if params.Player == pn then
                            self:stoptweening()
                            if lifeColor then
                                self:diffuse(lifeColor == 'rainbow' and HSV(colorCircle%360, 1, 1) or color(lifeColor))
                            elseif params.Life>=1.0 then
                                self:diffuse(tcConfig.Color.Hot)
                            else
                                self:diffuse(tcConfig.Color.Normal)
                            end
                            if params.Life >= 0.05*(i+1) then
                                self:croptop(0)
                            elseif params.Life < 0.05*i then
                                self:croptop(1)
                            else
                                self:croptop(1 - (params.Life - (0.05*i))*20)
                            end
                        end
                    end,
                    ScreenUpdateMessageCommand = function(self, params)
                        if lifeColor then
                            self:diffuse(lifeColor == 'rainbow' and HSV(colorCircle%360, 1, 1) or color(lifeColor))
                        end
                    end,
                })
                actors[#actors+1] = Def.Quad({
                    OnCommand = function(self)
                        self:zoomto(8, 32)
                        self:horizalign(left)
                        self:x(-128 + i*13)
                    end,
                    ScreenUpdateMessageCommand = function(self, params)
                        local white = 0.04*((params.PnBeat[pn]*5+(19-i))%20)
                        if pnLifeMax then
                            self:diffusealpha(0.6-math.abs(white-0.4)*1.5)
                        else
                            self:diffusealpha(0.8-white)
                        end
                    end,
                    PlayerLifeCommand = function(self, params)
                        if params.Player == pn then
                            self:stoptweening()
                            if params.Life >= 0.05*(i+1) then
                                self:croptop(0)
                            elseif params.Life < 0.05*i then
                                self:croptop(1)
                            else
                                self:croptop(1 - (params.Life - (0.05*i))*20)
                            end
                            pnLifeMax = params.Life>=1.0
                        end
                    end,
                })
            end
        end
        actors[#actors+1] = LoadActor('ya_life_frame')
        actors[#actors+1] = LoadFont('Common Normal')..{
            InitCommand = function(self)
                self:diffuse(1, 1, 1, 1)
                self:strokecolor(Color('Black'))
                self:skewx(-0.2)
                --self:horizalign(W:IsP1(pn) and left or right)
                self:xy(0, 0)
                self:zoom(0.75)
                self:rotationy(((pn == PLAYER_1) and 0 or 180))
                self:settext('')
            end,
            ChangedLifeTypeCommand = function(self, params)
                if not params.Level then return end
                if params.Player == pn then
                    lifeColor = params.Color
                    self:settext(params.Name or '')
                end
            end,
        }
        return actors
    end
    
    return Def.ActorFrame({
        OnCommand = function(self)
            self:xy(SCREEN_CENTER_X - WideScale(180, 210), SCREEN_TOP+32)
        end,
        NormalLife(),
        BatteryLife(),
    })
end

local function ScoreActor()
    local displayScore = 0
    return Def.ActorFrame({
        OnCommand = function(self)
            self:zoomx((pn == PLAYER_1) and 1 or -1)
            self:xy(SCREEN_CENTER_X-SCREEN_WIDTH/4+SCREEN_WIDTH/100-WideScale(50, 0), SCREEN_BOTTOM-30)
        end,
        LoadActor('score_frame'),
        LoadFont('waiei2 Score Numbers')..{
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:settext('')
                self:visible(W_PLAYER:IsEnabled(pn))
            end,
            ScreenUpdateMessageCommand = function(self, params)
                if not W_PLAYER:IsEnabled(pn) then return end
                local stats = STATSMAN:GetCurStageStats():GetPlayerStageStats(pn)
                if stats then
                    local currentScore = stats:GetScore()
                    if currentScore > displayScore then
                        displayScore = displayScore + math.ceil((currentScore - displayScore) / 2.5)
                    elseif currentScore < displayScore then
                        displayScore = currentScore
                    end
                    local formatScore = W_STRING:Number(displayScore)
                    local lenScore = string.len(''..displayScore)
                    self:settext(formatScore)
                    self:ClearAttributes()
                    self:AddAttribute(0, {
                        Length = math.max(string.len(formatScore) - (lenScore + math.floor((lenScore-1)/3)), 0),
                        Diffuse = color('#ffffff80'),
                    })
                end
            end,
        },
    })
end

local function DifficultyActor()
    if not W_PLAYER:IsEnabled(pn) then return Def.ActorFrame({}) end
    local difficulty
    return Def.ActorFrame({
        ChangedGameplaySongMessageCommand = function(self, params)
            local steps = W:Steps(pn)
            difficulty = steps:GetDifficulty()
            self:playcommand('ChangedPlayerDifficulty', {
                Player     = pn,
                Steps      = steps,
                Difficulty = difficulty,
            })
        end,
        LoadActor('difficulty')..{
            OnCommand = function(self)
                self:horizalign((pn == PLAYER_1) and left or right)
                self:vertalign(bottom)
                self:xy(0, SCREEN_BOTTOM - 42)
                self:zoomx((pn == PLAYER_1) and 1 or -1)
                self:animate(false)
            end,
            ChangedPlayerDifficultyMessageCommand = function(self, params)
                if params.Player == pn then
                    self:setstate(Enum.Reverse(Difficulty)[params.Difficulty]*2+((params.Player == PLAYER_1) and 0 or 1))
                end
            end,
        },
    })
end

return Def.ActorFrame({
    OnCommand = function(self)
        screen  = SCREENMAN:GetTopScreen()
        if W_PLAYER:IsEnabled(pn) then
            life    = screen:GetLifeMeter(pn) or nil
            battery = life.GetTotalLives and life:GetTotalLives() or nil
        else
            life    = screen:GetLifeMeter(W:APn(pn)) or nil
            battery = life.GetTotalLives and life:GetTotalLives() or nil
        end
        self:sleep(0.1)
        self:queuecommand('AfterOn')
    end,
    AfterOnCommand = function(self)
        local position = screen:GetChild('Overlay'):GetChild('Overlay'):GetChild('Position'):GetChild(pn)
        local graph = screen:GetChild('Underlay'):GetChild('Underlay'):GetChild('Note Graph'):GetChild(pn)
        if position then position:zoomy(0.87):y(32) end
        if graph then graph:zoomy(0.87):y(32) end
    end,
    -- 上フレーム
    LoadActor('frame_top')..{
        OnCommand = function(self)
            self:horizalign(right)
            self:vertalign(top)
            self:xy(SCREEN_CENTER_X, SCREEN_TOP)
            self:animate(false)
            self:setstate(2)
        end,
    },
    LoadActor('frame_top')..{
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:xy(0, SCREEN_TOP)
            self:animate(false)
            self:setstate(0)
        end,
    },
    LoadActor('frame_top')..{
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(top)
            self:xy(80, SCREEN_TOP)
            self:animate(false)
            self:setstate(1)
            self:zoomtowidth(SCREEN_WIDTH/2-160)
        end,
    },
    -- 下フレーム
    LoadActor('frame_bottom')..{
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(bottom)
            self:xy(0, SCREEN_BOTTOM)
            self:animate(false)
            self:setstate(0)
        end,
    },
    LoadActor('frame_bottom')..{
        OnCommand = function(self)
            self:horizalign(left)
            self:vertalign(bottom)
            self:xy(60, SCREEN_BOTTOM)
            self:animate(false)
            self:setstate(1)
            self:zoomtowidth(SCREEN_WIDTH/2-60)
        end,
    },
    -- 難易度
    DifficultyActor(),
    -- スコア
    ScoreActor(),
    -- ライフ
    LifeActor(),
    -- BPM
    LoadFont('Common Normal')..{
        OnCommand = function(self)
            self:horizalign(center)
            if GAMESTATE:GetNumPlayersEnabled() == 1 then
                self:x(SCREEN_CENTER_X)
            else
                self:x(SCREEN_CENTER_X-45)
            end
            self:y(SCREEN_BOTTOM-30)
            self:zoomx((pn == PLAYER_1) and 0.6 or -0.6)
            self:zoomy(0.6)
            self:maxwidth(120)
            self:diffuse(Color('White'))
            self:strokecolor(Color('Outline'))
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local steps  = W:Steps(pn)
            local timing = steps and steps:GetTimingData() or nil
            if timing then
                local beat = timing:GetBeatFromElapsedTime(params.Sec)
                local bpm  = timing:GetBPMAtBeat(beat)
                self:settextf('%4.1f', bpm)
            end
        end,
    },
    -- プレイヤー名
    LoadFont('Common Small Bold')..{
        Text = W_PLAYER:IsEnabled(pn) and W_PLAYER:Name(pn) or '',
        OnCommand = function(self)
            self:horizalign((pn == PLAYER_1) and left or right)
            self:vertalign(bottom)
            self:xy(5, SCREEN_BOTTOM-5)
            self:zoomx((pn == PLAYER_1) and 0.5 or -0.5)
            self:zoomy(0.5)
            self:strokecolor(Color('Black'))
        end,
    },
})